@@ -1,14 +1,14 @@
+import tempfile
 import sympy as sp
-from sympy.core.compatibility import exec_
 from sympy.codegen.ast import Assignment
 from sympy.codegen.algorithms import newtons_method, newtons_method_function
 from sympy.codegen.fnodes import bind_C
 from sympy.codegen.futils import render_as_module as f_module
 from sympy.codegen.pyutils import render_as_module as py_module
 from sympy.external import import_module
-from sympy.printing.ccode import ccode
+from sympy.printing.codeprinter import ccode
 from sympy.utilities._compilation import compile_link_import_strings, has_c, has_fortran
-from sympy.utilities._compilation.util import TemporaryDirectory, may_xfail
+from sympy.utilities._compilation.util import may_xfail
 from sympy.testing.pytest import skip, raises
 
 cython = import_module('cython')
@@ -33,7 +33,7 @@ def test_newtons_method_function__ccode():
         skip("No C compiler found.")
 
     compile_kw = dict(std='c99')
-    with TemporaryDirectory() as folder:
+    with tempfile.TemporaryDirectory() as folder:
         mod, info = compile_link_import_strings([
             ('newton.c', ('#include <math.h>\n'
                           '#include <stdio.h>\n') + ccode(func)),
@@ -57,7 +57,7 @@ def test_newtons_method_function__fcode():
         skip("No Fortran compiler found.")
 
     f_mod = f_module([func], 'mod_newton')
-    with TemporaryDirectory() as folder:
+    with tempfile.TemporaryDirectory() as folder:
         mod, info = compile_link_import_strings([
             ('newton.f90', f_mod),
             ('_newton.pyx', ("#cython: language_level={}\n".format("3") +
@@ -74,7 +74,7 @@ def test_newtons_method_function__pycode():
     func = newtons_method_function(expr, x)
     py_mod = py_module(func)
     namespace = {}
-    exec_(py_mod, namespace, namespace)
+    exec(py_mod, namespace, namespace)
     res = eval('newton(0.5)', namespace)
     assert abs(res - 0.865474033102) < 1e-12
 
@@ -94,7 +94,7 @@ def test_newtons_method_function__ccode_parameters():
         skip("cython not installed.")
 
     compile_kw = dict(std='c99')
-    with TemporaryDirectory() as folder:
+    with tempfile.TemporaryDirectory() as folder:
         mod, info = compile_link_import_strings([
             ('newton_par.c', ('#include <math.h>\n'
                           '#include <stdio.h>\n') + ccode(func)),
