@@ -1,18 +1,24 @@
-from sympy.core import Add, S, sympify, oo, pi, Dummy, expand_func
-from sympy.core.compatibility import as_int
-from sympy.core.function import Function, ArgumentIndexError
+from math import prod
+
+from sympy.core import Add, S, Dummy, expand_func
+from sympy.core.expr import Expr
+from sympy.core.function import Function, ArgumentIndexError, PoleError
 from sympy.core.logic import fuzzy_and, fuzzy_not
-from sympy.core.numbers import Rational
+from sympy.core.numbers import Rational, pi, oo, I
 from sympy.core.power import Pow
 from sympy.functions.special.zeta_functions import zeta
 from sympy.functions.special.error_functions import erf, erfc, Ei
-from sympy.functions.elementary.complexes import re
+from sympy.functions.elementary.complexes import re, unpolarify
 from sympy.functions.elementary.exponential import exp, log
 from sympy.functions.elementary.integers import ceiling, floor
 from sympy.functions.elementary.miscellaneous import sqrt
 from sympy.functions.elementary.trigonometric import sin, cos, cot
 from sympy.functions.combinatorial.numbers import bernoulli, harmonic
 from sympy.functions.combinatorial.factorials import factorial, rf, RisingFactorial
+from sympy.utilities.misc import as_int
+
+from mpmath import mp, workprec
+from mpmath.libmp.libmpf import prec_to_dps
 
 def intlike(n):
     try:
@@ -43,7 +49,7 @@ class gamma(Function):
     Examples
     ========
 
-    >>> from sympy import S, I, pi, oo, gamma
+    >>> from sympy import S, I, pi, gamma
     >>> from sympy.abc import x
 
     Several special values are known:
@@ -103,6 +109,7 @@ class gamma(Function):
     """
 
     unbranched = True
+    _singularities = (S.ComplexInfinity,)
 
     def fdiff(self, argindex=1):
         if argindex == 1:
@@ -136,8 +143,7 @@ class gamma(Function):
                         else:
                             coeff = S.NegativeOne
 
-                    for i in range(3, 2*k, 2):
-                        coeff *= i
+                    coeff *= prod(range(3, 2*k, 2))
 
                     if arg.is_positive:
                         return coeff*sqrt(S.Pi) / 2**n
@@ -183,41 +189,30 @@ class gamma(Function):
         elif x.is_noninteger:
             return floor(x).is_even
 
-    def _eval_rewrite_as_tractable(self, z, **kwargs):
+    def _eval_rewrite_as_tractable(self, z, limitvar=None, **kwargs):
         return exp(loggamma(z))
 
     def _eval_rewrite_as_factorial(self, z, **kwargs):
         return factorial(z - 1)
 
-    def _eval_nseries(self, x, n, logx):
+    def _eval_nseries(self, x, n, logx, cdir=0):
         x0 = self.args[0].limit(x, 0)
         if not (x0.is_Integer and x0 <= 0):
             return super()._eval_nseries(x, n, logx)
         t = self.args[0] - x0
         return (self.func(t + 1)/rf(self.args[0], -x0 + 1))._eval_nseries(x, n, logx)
 
-    def _sage_(self):
-        import sage.all as sage
-        return sage.gamma(self.args[0]._sage_())
-
-    def _eval_as_leading_term(self, x):
-        from sympy import Order
+    def _eval_as_leading_term(self, x, logx=None, cdir=0):
         arg = self.args[0]
-        arg_1 = arg.as_leading_term(x)
-        if Order(x, x).contains(arg_1):
-            return S(1) / arg_1
-        if Order(1, x).contains(arg_1):
-            return self.func(arg_1)
-        ####################################################
-        # The correct result here should be 'None'.        #
-        # Indeed arg in not bounded as x tends to 0.       #
-        # Consequently the series expansion does not admit #
-        # the leading term.                                #
-        # For compatibility reasons, the return value here #
-        # is the original function, i.e. gamma(arg),       #
-        # instead of None.                                 #
-        ####################################################
-        return self.func(arg)
+        x0 = arg.subs(x, 0)
+
+        if x0.is_integer and x0.is_nonpositive:
+            n = -x0
+            res = S.NegativeOne**n/self.func(n + 1)
+            return res/(arg + n).as_leading_term(x)
+        elif not x0.is_infinite:
+            return self.func(x0)
+        raise PoleError()
 
 
 ###############################################################################
@@ -281,7 +276,7 @@ class lowergamma(Function):
 
 
     def fdiff(self, argindex=2):
-        from sympy import meijerg, unpolarify
+        from sympy.functions.special.hyper import meijerg
         if argindex == 2:
             a, z = self.args
             return exp(-unpolarify(z))*z**(a - 1)
@@ -310,7 +305,6 @@ class lowergamma(Function):
         #    where lowergamma_unbranched(s, x) is an entire function (in fact
         #    of both s and x), i.e.
         #    lowergamma(s, exp(2*I*pi*n)*x) = exp(2*pi*I*n*a)*lowergamma(a, x)
-        from sympy import unpolarify, I
         if x is S.Zero:
             return S.Zero
         nx, n = x.extract_branch_factor()
@@ -320,7 +314,7 @@ class lowergamma(Function):
                 return lowergamma(a, nx)
         elif a.is_integer and a.is_nonpositive:
             if n != 0:
-                return 2*pi*I*n*(-1)**(-a)/factorial(-a) + lowergamma(a, nx)
+                return 2*pi*I*n*S.NegativeOne**(-a)/factorial(-a) + lowergamma(a, nx)
         elif n != 0:
             return exp(2*pi*I*n*a)*lowergamma(a, nx)
 
@@ -339,14 +333,12 @@ class lowergamma(Function):
                         return gamma(a)*(lowergamma(S.Half, x)/sqrt(pi) - exp(-x)*Add(*[x**(k - S.Half)/gamma(S.Half + k) for k in range(1, a + S.Half)]))
 
                 if not a.is_Integer:
-                    return (-1)**(S.Half - a)*pi*erf(sqrt(x))/gamma(1 - a) + exp(-x)*Add(*[x**(k + a - 1)*gamma(a)/gamma(a + k) for k in range(1, Rational(3, 2) - a)])
+                    return S.NegativeOne**(S.Half - a)*pi*erf(sqrt(x))/gamma(1 - a) + exp(-x)*Add(*[x**(k + a - 1)*gamma(a)/gamma(a + k) for k in range(1, Rational(3, 2) - a)])
 
         if x.is_zero:
             return S.Zero
 
     def _eval_evalf(self, prec):
-        from mpmath import mp, workprec
-        from sympy import Expr
         if all(x.is_number for x in self.args):
             a = self.args[0]._to_mpmath(prec)
             z = self.args[1]._to_mpmath(prec)
@@ -361,11 +353,38 @@ class lowergamma(Function):
         if x not in (S.Zero, S.NegativeInfinity):
             return self.func(self.args[0].conjugate(), x.conjugate())
 
+    def _eval_is_meromorphic(self, x, a):
+        # By https://en.wikipedia.org/wiki/Incomplete_gamma_function#Holomorphic_extension,
+        #    lowergamma(s, z) = z**s*gamma(s)*gammastar(s, z),
+        # where gammastar(s, z) is holomorphic for all s and z.
+        # Hence the singularities of lowergamma are z = 0  (branch
+        # point) and nonpositive integer values of s (poles of gamma(s)).
+        s, z = self.args
+        args_merom = fuzzy_and([z._eval_is_meromorphic(x, a),
+            s._eval_is_meromorphic(x, a)])
+        if not args_merom:
+            return args_merom
+        z0 = z.subs(x, a)
+        if s.is_integer:
+            return fuzzy_and([s.is_positive, z0.is_finite])
+        s0 = s.subs(x, a)
+        return fuzzy_and([s0.is_finite, z0.is_finite, fuzzy_not(z0.is_zero)])
+
+    def _eval_aseries(self, n, args0, x, logx):
+        from sympy.series.order import O
+        s, z = self.args
+        if args0[0] is S.Infinity and not z.has(x):
+            coeff = z**s*exp(-z)
+            sum_expr = sum(z**k/rf(s, k + 1) for k in range(n - 1))
+            o = O(z**s*s**(-n))
+            return coeff*sum_expr + o
+        return super()._eval_aseries(n, args0, x, logx)
+
     def _eval_rewrite_as_uppergamma(self, s, x, **kwargs):
         return gamma(s) - uppergamma(s, x)
 
     def _eval_rewrite_as_expint(self, s, x, **kwargs):
-        from sympy import expint
+        from sympy.functions.special.error_functions import expint
         if s.is_integer and s.is_nonpositive:
             return self
         return self.rewrite(uppergamma).rewrite(expint)
@@ -443,7 +462,7 @@ class uppergamma(Function):
 
 
     def fdiff(self, argindex=2):
-        from sympy import meijerg, unpolarify
+        from sympy.functions.special.hyper import meijerg
         if argindex == 2:
             a, z = self.args
             return -exp(-unpolarify(z))*z**(a - 1)
@@ -454,8 +473,6 @@ class uppergamma(Function):
             raise ArgumentIndexError(self, argindex)
 
     def _eval_evalf(self, prec):
-        from mpmath import mp, workprec
-        from sympy import Expr
         if all(x.is_number for x in self.args):
             a = self.args[0]._to_mpmath(prec)
             z = self.args[1]._to_mpmath(prec)
@@ -466,7 +483,7 @@ class uppergamma(Function):
 
     @classmethod
     def eval(cls, a, z):
-        from sympy import unpolarify, I, expint
+        from sympy.functions.special.error_functions import expint
         if z.is_Number:
             if z is S.NaN:
                 return S.NaN
@@ -484,7 +501,7 @@ class uppergamma(Function):
                 return uppergamma(a, nx)
         elif a.is_integer and a.is_nonpositive:
             if n != 0:
-                return -2*pi*I*n*(-1)**(-a)/factorial(-a) + uppergamma(a, nx)
+                return -2*pi*I*n*S.NegativeOne**(-a)/factorial(-a) + uppergamma(a, nx)
         elif n != 0:
             return gamma(a)*(1 - exp(2*pi*I*n*a)) + exp(2*pi*I*n*a)*uppergamma(a, nx)
 
@@ -500,14 +517,20 @@ class uppergamma(Function):
                 b = a - 1
                 if b.is_positive:
                     if a.is_integer:
-                        return exp(-z) * factorial(b) * Add(*[z**k / factorial(k) for k in range(a)])
+                        return exp(-z) * factorial(b) * Add(*[z**k / factorial(k)
+                                                              for k in range(a)])
                     else:
-                        return gamma(a) * erfc(sqrt(z)) + (-1)**(a - S(3)/2) * exp(-z) * sqrt(z) * Add(*[gamma(-S.Half - k) * (-z)**k / gamma(1-a) for k in range(a - S.Half)])
+                        return (gamma(a) * erfc(sqrt(z)) +
+                                S.NegativeOne**(a - S(3)/2) * exp(-z) * sqrt(z)
+                                * Add(*[gamma(-S.Half - k) * (-z)**k / gamma(1-a)
+                                        for k in range(a - S.Half)]))
                 elif b.is_Integer:
                     return expint(-b, z)*unpolarify(z)**(b + 1)
 
                 if not a.is_Integer:
-                    return (-1)**(S.Half - a) * pi*erfc(sqrt(z))/gamma(1-a) - z**a * exp(-z) * Add(*[z**k * gamma(a) / gamma(a+k+1) for k in range(S.Half - a)])
+                    return (S.NegativeOne**(S.Half - a) * pi*erfc(sqrt(z))/gamma(1-a)
+                            - z**a * exp(-z) * Add(*[z**k * gamma(a) / gamma(a+k+1)
+                                                     for k in range(S.Half - a)]))
 
         if a.is_zero and z.is_positive:
             return -Ei(-z)
@@ -517,19 +540,21 @@ class uppergamma(Function):
 
     def _eval_conjugate(self):
         z = self.args[1]
-        if not z in (S.Zero, S.NegativeInfinity):
+        if z not in (S.Zero, S.NegativeInfinity):
             return self.func(self.args[0].conjugate(), z.conjugate())
 
+    def _eval_is_meromorphic(self, x, a):
+        return lowergamma._eval_is_meromorphic(self, x, a)
+
     def _eval_rewrite_as_lowergamma(self, s, x, **kwargs):
         return gamma(s) - lowergamma(s, x)
 
-    def _eval_rewrite_as_expint(self, s, x, **kwargs):
-        from sympy import expint
-        return expint(1 - s, x)*x**s
+    def _eval_rewrite_as_tractable(self, s, x, **kwargs):
+        return exp(loggamma(s)) - lowergamma(s, x)
 
-    def _sage_(self):
-        import sage.all as sage
-        return sage.gamma(self.args[0]._sage_(), self.args[1]._sage_())
+    def _eval_rewrite_as_expint(self, s, x, **kwargs):
+        from sympy.functions.special.error_functions import expint
+        return expint(1 - s, x)*x**s
 
 
 ###############################################################################
@@ -667,7 +692,7 @@ class polygamma(Function):
             return self.args[0].is_even
 
     def _eval_aseries(self, n, args0, x, logx):
-        from sympy import Order
+        from sympy.series.order import Order
         if args0[1] != oo or not \
                 (self.args[0].is_Integer and self.args[0].is_nonnegative):
             return super()._eval_aseries(n, args0, x, logx)
@@ -685,7 +710,7 @@ class polygamma(Function):
                 m = ceiling((n + 1)//2)
                 l = [bernoulli(2*k) / (2*k*z**(2*k)) for k in range(1, m)]
                 r -= Add(*l)
-                o = Order(1/z**(2*m), x)
+                o = Order(1/z**n, x)
             return r._eval_nseries(x, n, logx) + o
         else:
             # proper polygamma function
@@ -709,9 +734,6 @@ class polygamma(Function):
 
     @classmethod
     def eval(cls, n, z):
-        n, z = map(sympify, (n, z))
-        from sympy import unpolarify
-
         if n.is_integer:
             if n.is_nonnegative:
                 nz = unpolarify(z)
@@ -720,7 +742,7 @@ class polygamma(Function):
 
             if n.is_positive:
                 if z is S.Half:
-                    return (-1)**(n + 1)*factorial(n)*(2**(n + 1) - 1)*zeta(n + 1)
+                    return S.NegativeOne**(n + 1)*factorial(n)*(2**(n + 1) - 1)*zeta(n + 1)
 
             if n is S.NegativeOne:
                 return loggamma(z)
@@ -741,9 +763,9 @@ class polygamma(Function):
                             return S.ComplexInfinity
                         else:
                             if n.is_zero:
-                                return -S.EulerGamma + harmonic(z - 1, 1)
+                                return harmonic(z - 1, 1) - S.EulerGamma
                             elif n.is_odd:
-                                return (-1)**(n + 1)*factorial(n)*zeta(n + 1, z)
+                                return S.NegativeOne**(n + 1)*factorial(n)*zeta(n + 1, z)
 
         if n.is_zero:
             if z is S.NaN:
@@ -778,14 +800,14 @@ class polygamma(Function):
                             z - i, e) for i in range(1, int(coeff) + 1)])
                     else:
                         tail = -Add(*[Pow(
-                            z + i, e) for i in range(0, int(-coeff))])
-                    return polygamma(n, z - coeff) + (-1)**n*factorial(n)*tail
+                            z + i, e) for i in range(int(-coeff))])
+                    return polygamma(n, z - coeff) + S.NegativeOne**n*factorial(n)*tail
 
             elif z.is_Mul:
                 coeff, z = z.as_two_terms()
                 if coeff.is_Integer and coeff.is_positive:
-                    tail = [ polygamma(n, z + Rational(
-                        i, coeff)) for i in range(0, int(coeff)) ]
+                    tail = [polygamma(n, z + Rational(
+                        i, coeff)) for i in range(int(coeff))]
                     if n == 0:
                         return Add(*tail)/coeff + log(coeff)
                     else:
@@ -814,7 +836,7 @@ class polygamma(Function):
     def _eval_rewrite_as_zeta(self, n, z, **kwargs):
         if n.is_integer:
             if (n - S.One).is_nonnegative:
-                return (-1)**(n + 1)*factorial(n)*zeta(n + 1, z)
+                return S.NegativeOne**(n + 1)*factorial(n)*zeta(n + 1, z)
 
     def _eval_rewrite_as_harmonic(self, n, z, **kwargs):
         if n.is_integer:
@@ -823,8 +845,8 @@ class polygamma(Function):
             else:
                 return S.NegativeOne**(n+1) * factorial(n) * (zeta(n+1) - harmonic(z-1, n+1))
 
-    def _eval_as_leading_term(self, x):
-        from sympy import Order
+    def _eval_as_leading_term(self, x, logx=None, cdir=0):
+        from sympy.series.order import Order
         n, z = [a.as_leading_term(x) for a in self.args]
         o = Order(z, x)
         if n == 0 and o.contains(1/x):
@@ -867,7 +889,7 @@ class loggamma(Function):
 
     For half-integral values:
 
-    >>> from sympy import S, pi
+    >>> from sympy import S
     >>> loggamma(S(5)/2)
     log(3*sqrt(pi)/4)
     >>> loggamma(n/2)
@@ -911,7 +933,7 @@ class loggamma(Function):
     Series expansion is also supported:
 
     >>> from sympy import series
-    >>> series(loggamma(x), x, 0, 4)
+    >>> series(loggamma(x), x, 0, 4).cancel()
     -log(x) - EulerGamma*x + pi**2*x**2/12 + x**3*polygamma(2, 1)/6 + O(x**4)
 
     We can numerically evaluate the ``gamma`` function to arbitrary precision
@@ -945,8 +967,6 @@ class loggamma(Function):
     """
     @classmethod
     def eval(cls, z):
-        z = sympify(z)
-
         if z.is_integer:
             if z.is_nonpositive:
                 return S.Infinity
@@ -966,7 +986,7 @@ class loggamma(Function):
             return S.NaN
 
     def _eval_expand_func(self, **hints):
-        from sympy import Sum
+        from sympy.concrete.summations import Sum
         z = self.args[0]
 
         if z.is_Rational:
@@ -986,7 +1006,7 @@ class loggamma(Function):
 
         return self
 
-    def _eval_nseries(self, x, n, logx=None):
+    def _eval_nseries(self, x, n, logx=None, cdir=0):
         x0 = self.args[0].limit(x, 0)
         if x0.is_zero:
             f = self._eval_rewrite_as_intractable(*self.args)
@@ -994,18 +1014,17 @@ class loggamma(Function):
         return super()._eval_nseries(x, n, logx)
 
     def _eval_aseries(self, n, args0, x, logx):
-        from sympy import Order
+        from sympy.series.order import Order
         if args0[0] != oo:
             return super()._eval_aseries(n, args0, x, logx)
         z = self.args[0]
-        m = min(n, ceiling((n + S.One)/2))
         r = log(z)*(z - S.Half) - z + log(2*pi)/2
-        l = [bernoulli(2*k) / (2*k*(2*k - 1)*z**(2*k - 1)) for k in range(1, m)]
+        l = [bernoulli(2*k) / (2*k*(2*k - 1)*z**(2*k - 1)) for k in range(1, n)]
         o = None
-        if m == 0:
+        if n == 0:
             o = Order(1, x)
         else:
-            o = Order(1/z**(2*m - 1), x)
+            o = Order(1/z**n, x)
         # It is very inefficient to first add the order and then do the nseries
         return (r + Add(*l))._eval_nseries(x, n, logx) + o
 
@@ -1021,7 +1040,7 @@ class loggamma(Function):
 
     def _eval_conjugate(self):
         z = self.args[0]
-        if not z in (S.Zero, S.NegativeInfinity):
+        if z not in (S.Zero, S.NegativeInfinity):
             return self.func(z.conjugate())
 
     def fdiff(self, argindex=1):
@@ -1030,10 +1049,6 @@ class loggamma(Function):
         else:
             raise ArgumentIndexError(self, argindex)
 
-    def _sage_(self):
-        import sage.all as sage
-        return sage.log_gamma(self.args[0]._sage_())
-
 
 class digamma(Function):
     r"""
@@ -1085,7 +1100,8 @@ class digamma(Function):
     """
     def _eval_evalf(self, prec):
         z = self.args[0]
-        return polygamma(0, z).evalf(prec)
+        nprec = prec_to_dps(prec)
+        return polygamma(0, z).evalf(n=nprec)
 
     def fdiff(self, argindex=1):
         z = self.args[0]
@@ -1122,7 +1138,7 @@ class digamma(Function):
     def _eval_rewrite_as_polygamma(self, z, **kwargs):
         return polygamma(0, z)
 
-    def _eval_as_leading_term(self, x):
+    def _eval_as_leading_term(self, x, logx=None, cdir=0):
         z = self.args[0]
         return polygamma(0, z).as_leading_term(x)
 
@@ -1178,7 +1194,8 @@ class trigamma(Function):
     """
     def _eval_evalf(self, prec):
         z = self.args[0]
-        return polygamma(1, z).evalf(prec)
+        nprec = prec_to_dps(prec)
+        return polygamma(1, z).evalf(n=nprec)
 
     def fdiff(self, argindex=1):
         z = self.args[0]
@@ -1218,7 +1235,7 @@ class trigamma(Function):
     def _eval_rewrite_as_harmonic(self, z, **kwargs):
         return -harmonic(z - 1, 2) + S.Pi**2 / 6
 
-    def _eval_as_leading_term(self, x):
+    def _eval_as_leading_term(self, x, logx=None, cdir=0):
         z = self.args[0]
         return polygamma(1, z).as_leading_term(x)
 
@@ -1240,7 +1257,7 @@ class multigamma(Function):
     Examples
     ========
 
-    >>> from sympy import S, I, pi, oo, gamma, multigamma
+    >>> from sympy import S, multigamma
     >>> from sympy import Symbol
     >>> x = Symbol('x')
     >>> p = Symbol('p', positive=True, integer=True)
@@ -1288,7 +1305,7 @@ class multigamma(Function):
     unbranched = True
 
     def fdiff(self, argindex=2):
-        from sympy import Sum
+        from sympy.concrete.summations import Sum
         if argindex == 2:
             x, p = self.args
             k = Dummy("k")
@@ -1298,8 +1315,7 @@ class multigamma(Function):
 
     @classmethod
     def eval(cls, x, p):
-        from sympy import Product
-        x, p = map(sympify, (x, p))
+        from sympy.concrete.products import Product
         if p.is_positive is False or p.is_integer is False:
             raise ValueError('Order parameter p must be positive integer.')
         k = Dummy("k")
