@@ -1,11 +1,9 @@
-from sympy import Integer
+from sympy.core.containers import Tuple
+from sympy.core.numbers import (Integer, Rational)
+from sympy.core.singleton import S
 import sympy.polys
 
-import sys
-if sys.version_info < (3,5):
-    from fractions import gcd
-else:
-    from math import gcd
+from math import gcd
 
 
 def egyptian_fraction(r, algorithm="Greedy"):
@@ -16,8 +14,8 @@ def egyptian_fraction(r, algorithm="Greedy"):
     Parameters
     ==========
 
-    r : Rational
-        a positive rational number.
+    r : Rational or (p, q)
+        a positive rational number, ``p/q``.
     algorithm : { "Greedy", "Graham Jewett", "Takenouchi", "Golomb" }, optional
         Denotes the algorithm to be used (the default is "Greedy").
 
@@ -28,13 +26,13 @@ def egyptian_fraction(r, algorithm="Greedy"):
     >>> from sympy.ntheory.egyptian_fraction import egyptian_fraction
     >>> egyptian_fraction(Rational(3, 7))
     [3, 11, 231]
-    >>> egyptian_fraction(Rational(3, 7), "Graham Jewett")
+    >>> egyptian_fraction((3, 7), "Graham Jewett")
     [7, 8, 9, 56, 57, 72, 3192]
-    >>> egyptian_fraction(Rational(3, 7), "Takenouchi")
+    >>> egyptian_fraction((3, 7), "Takenouchi")
     [4, 7, 28]
-    >>> egyptian_fraction(Rational(3, 7), "Golomb")
+    >>> egyptian_fraction((3, 7), "Golomb")
     [3, 15, 35]
-    >>> egyptian_fraction(Rational(11, 5), "Golomb")
+    >>> egyptian_fraction((11, 5), "Golomb")
     [1, 2, 3, 4, 9, 234, 1118, 2580]
 
     See Also
@@ -106,31 +104,47 @@ def egyptian_fraction(r, algorithm="Greedy"):
 
     """
 
+    if not isinstance(r, Rational):
+        if isinstance(r, (Tuple, tuple)) and len(r) == 2:
+            r = Rational(*r)
+        else:
+            raise ValueError("Value must be a Rational or tuple of ints")
     if r <= 0:
         raise ValueError("Value must be positive")
 
+    # common cases that all methods agree on
+    x, y = r.as_numer_denom()
+    if y == 1 and x == 2:
+        return [Integer(i) for i in [1, 2, 3, 6]]
+    if x == y + 1:
+        return [S.One, y]
+
     prefix, rem = egypt_harmonic(r)
     if rem == 0:
         return prefix
-    x, y = rem.as_numer_denom()
+    # work in Python ints
+    x, y = rem.p, rem.q
+    # assert x < y and gcd(x, y) = 1
 
     if algorithm == "Greedy":
-        return prefix + egypt_greedy(x, y)
+        postfix = egypt_greedy(x, y)
     elif algorithm == "Graham Jewett":
-        return prefix + egypt_graham_jewett(x, y)
+        postfix = egypt_graham_jewett(x, y)
     elif algorithm == "Takenouchi":
-        return prefix + egypt_takenouchi(x, y)
+        postfix = egypt_takenouchi(x, y)
     elif algorithm == "Golomb":
-        return prefix + egypt_golomb(x, y)
+        postfix = egypt_golomb(x, y)
     else:
         raise ValueError("Entered invalid algorithm")
+    return prefix + [Integer(i) for i in postfix]
 
 
 def egypt_greedy(x, y):
+    # assumes gcd(x, y) == 1
     if x == 1:
         return [y]
     else:
-        a = (-y) % (x)
+        a = (-y) % x
         b = y*(y//x + 1)
         c = gcd(a, b)
         if c > 1:
@@ -141,6 +155,7 @@ def egypt_greedy(x, y):
 
 
 def egypt_graham_jewett(x, y):
+    # assumes gcd(x, y) == 1
     l = [y] * x
 
     # l is now a list of integers whose reciprocals sum to x/y.
@@ -162,6 +177,15 @@ def egypt_graham_jewett(x, y):
 
 
 def egypt_takenouchi(x, y):
+    # assumes gcd(x, y) == 1
+    # special cases for 3/y
+    if x == 3:
+        if y % 2 == 0:
+            return [y//2, y]
+        i = (y - 1)//2
+        j = i + 1
+        k = j + i
+        return [j, k, j*k]
     l = [y] * x
     while len(l) != len(set(l)):
         l.sort()
@@ -178,18 +202,20 @@ def egypt_takenouchi(x, y):
 
 
 def egypt_golomb(x, y):
+    # assumes x < y and gcd(x, y) == 1
     if x == 1:
         return [y]
     xp = sympy.polys.ZZ.invert(int(x), int(y))
-    rv = [Integer(xp*y)]
+    rv = [xp*y]
     rv.extend(egypt_golomb((x*xp - 1)//y, xp))
     return sorted(rv)
 
 
 def egypt_harmonic(r):
+    # assumes r is Rational
     rv = []
-    d = Integer(1)
-    acc = Integer(0)
+    d = S.One
+    acc = S.Zero
     while acc + 1/d <= r:
         acc += 1/d
         rv.append(d)
