@@ -1,11 +1,16 @@
-from sympy import S, Integral, sin, cos, pi, sqrt, symbols
+from sympy.core.numbers import pi
+from sympy.core.singleton import S
+from sympy.core.symbol import symbols
+from sympy.functions.elementary.miscellaneous import sqrt
+from sympy.functions.elementary.trigonometric import (cos, sin)
+from sympy.integrals.integrals import Integral
 from sympy.physics.vector import Dyadic, Point, ReferenceFrame, Vector
 from sympy.physics.vector.functions import (cross, dot, express,
                                             time_derivative,
                                             kinematic_equations, outer,
                                             partial_velocity,
                                             get_motion_params, dynamicsymbols)
-from sympy.utilities.pytest import raises
+from sympy.testing.pytest import raises
 
 Vector.simp = True
 q1, q2, q3, q4, q5 = symbols('q1 q2 q3 q4 q5')
@@ -108,15 +113,15 @@ def test_operator_match():
     assert v & d == dot(v, d)
     assert d & zerov == dot(d, zerov)
     assert zerov & d == dot(zerov, d)
-    raises(TypeError, lambda: dot(d, S(0)))
-    raises(TypeError, lambda: dot(S(0), d))
+    raises(TypeError, lambda: dot(d, S.Zero))
+    raises(TypeError, lambda: dot(S.Zero, d))
     raises(TypeError, lambda: dot(d, 0))
     raises(TypeError, lambda: dot(0, d))
     assert v & v == dot(v, v)
     assert v & zerov == dot(v, zerov)
     assert zerov & v == dot(zerov, v)
-    raises(TypeError, lambda: dot(v, S(0)))
-    raises(TypeError, lambda: dot(S(0), v))
+    raises(TypeError, lambda: dot(v, S.Zero))
+    raises(TypeError, lambda: dot(S.Zero, v))
     raises(TypeError, lambda: dot(v, 0))
     raises(TypeError, lambda: dot(0, v))
 
@@ -129,15 +134,15 @@ def test_operator_match():
     assert d ^ zerov == cross(d, zerov)
     assert zerov ^ d == cross(zerov, d)
     assert zerov ^ d == cross(zerov, d)
-    raises(TypeError, lambda: cross(d, S(0)))
-    raises(TypeError, lambda: cross(S(0), d))
+    raises(TypeError, lambda: cross(d, S.Zero))
+    raises(TypeError, lambda: cross(S.Zero, d))
     raises(TypeError, lambda: cross(d, 0))
     raises(TypeError, lambda: cross(0, d))
     assert v ^ v == cross(v, v)
     assert v ^ zerov == cross(v, zerov)
     assert zerov ^ v == cross(zerov, v)
-    raises(TypeError, lambda: cross(v, S(0)))
-    raises(TypeError, lambda: cross(S(0), v))
+    raises(TypeError, lambda: cross(v, S.Zero))
+    raises(TypeError, lambda: cross(S.Zero, v))
     raises(TypeError, lambda: cross(v, 0))
     raises(TypeError, lambda: cross(0, v))
 
@@ -150,22 +155,22 @@ def test_operator_match():
     raises(TypeError, lambda: outer(d, zerov))
     raises(TypeError, lambda: outer(zerov, d))
     raises(TypeError, lambda: outer(zerov, d))
-    raises(TypeError, lambda: outer(d, S(0)))
-    raises(TypeError, lambda: outer(S(0), d))
+    raises(TypeError, lambda: outer(d, S.Zero))
+    raises(TypeError, lambda: outer(S.Zero, d))
     raises(TypeError, lambda: outer(d, 0))
     raises(TypeError, lambda: outer(0, d))
     assert v | v == outer(v, v)
     assert v | zerov == outer(v, zerov)
     assert zerov | v == outer(zerov, v)
-    raises(TypeError, lambda: outer(v, S(0)))
-    raises(TypeError, lambda: outer(S(0), v))
+    raises(TypeError, lambda: outer(v, S.Zero))
+    raises(TypeError, lambda: outer(S.Zero, v))
     raises(TypeError, lambda: outer(v, 0))
     raises(TypeError, lambda: outer(0, v))
 
 
 def test_express():
     assert express(Vector(0), N) == Vector(0)
-    assert express(S(0), N) == S(0)
+    assert express(S.Zero, N) is S.Zero
     assert express(A.x, C) == cos(q3)*C.x + sin(q3)*C.z
     assert express(A.y, C) == sin(q2)*sin(q3)*C.x + cos(q2)*C.y - \
         sin(q2)*cos(q3)*C.z
@@ -488,3 +493,16 @@ def test_partial_velocity():
 
     raises(TypeError, lambda: partial_velocity(Dmc.vel(N), u_list, N))
     raises(TypeError, lambda: partial_velocity(vel_list, u1, N))
+
+def test_dynamicsymbols():
+    #Tests to check the assumptions applied to dynamicsymbols
+    f1 = dynamicsymbols('f1')
+    f2 = dynamicsymbols('f2', real=True)
+    f3 = dynamicsymbols('f3', positive=True)
+    f4, f5 = dynamicsymbols('f4,f5', commutative=False)
+    f6 = dynamicsymbols('f6', integer=True)
+    assert f1.is_real is None
+    assert f2.is_real
+    assert f3.is_positive
+    assert f4*f5 != f5*f4
+    assert f6.is_integer
