@@ -1,9 +1,12 @@
-from sympy.core.backend import (S, sympify, expand, sqrt, Add, zeros,
-    ImmutableMatrix as Matrix)
-from sympy import trigsimp
+from sympy.core.backend import (S, sympify, expand, sqrt, Add, zeros, acos,
+                                ImmutableMatrix as Matrix, _simplify_matrix)
+from sympy.simplify.trigsimp import trigsimp
 from sympy.printing.defaults import Printable
 from sympy.utilities.misc import filldedent
-from sympy.core.evalf import EvalfMixin, prec_to_dps
+from sympy.core.evalf import EvalfMixin
+
+from mpmath.libmp.libmpf import prec_to_dps
+
 
 __all__ = ['Vector']
 
@@ -26,7 +29,7 @@ class Vector(Printable, EvalfMixin):
     is_number = False
 
     def __init__(self, inlist):
-        """This is the constructor for the Vector class.  You shouldn't be
+        """This is the constructor for the Vector class.  You should not be
         calling this, it should only be used by other functions. You should be
         treating Vectors like you would with if you were doing the math by
         hand, and getting the first 3 from the standard basis vectors from a
@@ -108,13 +111,13 @@ class Vector(Printable, EvalfMixin):
                         * (v2[1].dcm(v1[1]))
                         * (v1[0]))[0]
         if Vector.simp:
-            return trigsimp(sympify(out), recursive=True)
+            return trigsimp(out, recursive=True)
         else:
-            return sympify(out)
+            return out
 
     def __truediv__(self, other):
         """This uses mul and inputs self and 1 divided by other. """
-        return self.__mul__(sympify(1) / other)
+        return self.__mul__(S.One / other)
 
     def __eq__(self, other):
         """Tests for equality.
@@ -168,13 +171,11 @@ class Vector(Printable, EvalfMixin):
         """
 
         newlist = [v for v in self.args]
+        other = sympify(other)
         for i, v in enumerate(newlist):
-            newlist[i] = (sympify(other) * newlist[i][0], newlist[i][1])
+            newlist[i] = (other * newlist[i][0], newlist[i][1])
         return Vector(newlist)
 
-    def __ne__(self, other):
-        return not self == other
-
     def __neg__(self):
         return self * -1
 
@@ -284,8 +285,8 @@ class Vector(Printable, EvalfMixin):
                                 tmp = pform.parens()
                                 pform = prettyForm(tmp[0], tmp[1])
 
-                            pform = prettyForm(*pform.right(" ",
-                                                ar[i][1].pretty_vecs[j]))
+                            pform = prettyForm(*pform.right(
+                                " ", ar[i][1].pretty_vecs[j]))
                         else:
                             continue
                         pforms.append(pform)
@@ -400,16 +401,17 @@ class Vector(Printable, EvalfMixin):
         Examples
         ========
 
-        >>> from sympy.physics.vector import ReferenceFrame
         >>> from sympy import symbols
+        >>> from sympy.physics.vector import ReferenceFrame, cross
         >>> q1 = symbols('q1')
         >>> N = ReferenceFrame('N')
-        >>> N.x ^ N.y
+        >>> cross(N.x, N.y)
         N.z
-        >>> A = N.orientnew('A', 'Axis', [q1, N.x])
-        >>> A.x ^ N.y
+        >>> A = ReferenceFrame('A')
+        >>> A.orient_axis(N, q1, N.x)
+        >>> cross(A.x, N.y)
         N.z
-        >>> N.y ^ A.x
+        >>> cross(N.y, A.x)
         - sin(q1)*A.y - cos(q1)*A.z
 
         """
@@ -424,8 +426,8 @@ class Vector(Printable, EvalfMixin):
         def _det(mat):
             """This is needed as a little method for to find the determinant
             of a list in python; needs to work for a 3x3 list.
-            SymPy's Matrix won't take in Vector, so need a custom function.
-            You shouldn't be calling this.
+            SymPy's Matrix will not take in Vector, so need a custom function.
+            You should not be calling this.
 
             """
 
@@ -440,9 +442,10 @@ class Vector(Printable, EvalfMixin):
             tempx = v[1].x
             tempy = v[1].y
             tempz = v[1].z
-            tempm = ([[tempx, tempy, tempz], [self & tempx, self & tempy,
-                self & tempz], [Vector([ar[i]]) & tempx,
-                Vector([ar[i]]) & tempy, Vector([ar[i]]) & tempz]])
+            tempm = ([[tempx, tempy, tempz],
+                      [self & tempx, self & tempy, self & tempz],
+                      [Vector([ar[i]]) & tempx, Vector([ar[i]]) & tempy,
+                       Vector([ar[i]]) & tempz]])
             outlist += _det(tempm).args
         return Vector(outlist)
 
@@ -519,6 +522,8 @@ class Vector(Printable, EvalfMixin):
         >>> N = ReferenceFrame('N')
         >>> A = N.orientnew('A', 'Axis', [q1, N.y])
         >>> A.x.diff(t, N)
+        - sin(q1)*q1'*N.x - cos(q1)*q1'*N.z
+        >>> A.x.diff(t, N).express(A)
         - q1'*A.z
         >>> B = ReferenceFrame('B')
         >>> u1, u2 = dynamicsymbols('u1, u2')
@@ -530,8 +535,8 @@ class Vector(Printable, EvalfMixin):
 
         from sympy.physics.vector.frame import _check_frame
 
-        var = sympify(var)
         _check_frame(frame)
+        var = sympify(var)
 
         inlist = []
 
@@ -545,12 +550,11 @@ class Vector(Printable, EvalfMixin):
                 # with the derivative frame does not contain the variable.
                 if not var_in_dcm or (frame.dcm(component_frame).diff(var) ==
                                       zeros(3, 3)):
-                    inlist += [(measure_number.diff(var),
-                                        component_frame)]
+                    inlist += [(measure_number.diff(var), component_frame)]
                 else:  # else express in the frame
                     reexp_vec_comp = Vector([vector_component]).express(frame)
                     deriv = reexp_vec_comp.args[0][0].diff(var)
-                    inlist += Vector([(deriv, frame)]).express(component_frame).args
+                    inlist += Vector([(deriv, frame)]).args
 
         return Vector(inlist)
 
@@ -653,7 +657,7 @@ class Vector(Printable, EvalfMixin):
         """Returns a simplified Vector."""
         d = {}
         for v in self.args:
-            d[v[1]] = v[0].simplify()
+            d[v[1]] = _simplify_matrix(v[0])
         return Vector(d)
 
     def subs(self, *args, **kwargs):
@@ -678,7 +682,17 @@ class Vector(Printable, EvalfMixin):
         return Vector(d)
 
     def magnitude(self):
-        """Returns the magnitude (Euclidean norm) of self."""
+        """Returns the magnitude (Euclidean norm) of self.
+
+        Warnings
+        ========
+
+        Python ignores the leading negative sign so that might
+        give wrong results.
+        ``-A.x.magnitude()`` would be treated as ``-(A.x.magnitude())``,
+        instead of ``(-A.x).magnitude()``.
+
+        """
         return sqrt(self & self)
 
     def normalize(self):
@@ -695,33 +709,99 @@ class Vector(Printable, EvalfMixin):
             d[v[1]] = v[0].applyfunc(f)
         return Vector(d)
 
-    def free_symbols(self, reference_frame):
+    def angle_between(self, vec):
         """
-        Returns the free symbols in the measure numbers of the vector
-        expressed in the given reference frame.
+        Returns the smallest angle between Vector 'vec' and self.
 
         Parameter
         =========
 
+        vec : Vector
+            The Vector between which angle is needed.
+
+        Examples
+        ========
+
+        >>> from sympy.physics.vector import ReferenceFrame
+        >>> A = ReferenceFrame("A")
+        >>> v1 = A.x
+        >>> v2 = A.y
+        >>> v1.angle_between(v2)
+        pi/2
+
+        >>> v3 = A.x + A.y + A.z
+        >>> v1.angle_between(v3)
+        acos(sqrt(3)/3)
+
+        Warnings
+        ========
+
+        Python ignores the leading negative sign so that might give wrong
+        results. ``-A.x.angle_between()`` would be treated as
+        ``-(A.x.angle_between())``, instead of ``(-A.x).angle_between()``.
+
+        """
+
+        vec1 = self.normalize()
+        vec2 = vec.normalize()
+        angle = acos(vec1.dot(vec2))
+        return angle
+
+    def free_symbols(self, reference_frame):
+        """Returns the free symbols in the measure numbers of the vector
+        expressed in the given reference frame.
+
+        Parameters
+        ==========
         reference_frame : ReferenceFrame
-            The frame with respect to which the free symbols of the
-            given vector is to be determined.
+            The frame with respect to which the free symbols of the given
+            vector is to be determined.
+
+        Returns
+        =======
+        set of Symbol
+            set of symbols present in the measure numbers of
+            ``reference_frame``.
 
         """
 
         return self.to_matrix(reference_frame).free_symbols
 
+    def free_dynamicsymbols(self, reference_frame):
+        """Returns the free dynamic symbols (functions of time ``t``) in the
+        measure numbers of the vector expressed in the given reference frame.
+
+        Parameters
+        ==========
+        reference_frame : ReferenceFrame
+            The frame with respect to which the free dynamic symbols of the
+            given vector is to be determined.
+
+        Returns
+        =======
+        set
+            Set of functions of time ``t``, e.g.
+            ``Function('f')(me.dynamicsymbols._t)``.
+
+        """
+        # TODO : Circular dependency if imported at top. Should move
+        # find_dynamicsymbols into physics.vector.functions.
+        from sympy.physics.mechanics.functions import find_dynamicsymbols
+
+        return find_dynamicsymbols(self, reference_frame=reference_frame)
+
     def _eval_evalf(self, prec):
         if not self.args:
             return self
         new_args = []
+        dps = prec_to_dps(prec)
         for mat, frame in self.args:
-            new_args.append([mat.evalf(n=prec_to_dps(prec)), frame])
+            new_args.append([mat.evalf(n=dps), frame])
         return Vector(new_args)
 
     def xreplace(self, rule):
-        """
-        Replace occurrences of objects within the measure numbers of the vector.
+        """Replace occurrences of objects within the measure numbers of the
+        vector.
 
         Parameters
         ==========
@@ -763,6 +843,7 @@ class Vector(Printable, EvalfMixin):
             new_args.append([mat, frame])
         return Vector(new_args)
 
+
 class VectorTypeError(TypeError):
 
     def __init__(self, other, want):
