@@ -1,4 +1,7 @@
-"""Implicit plotting module for SymPy
+"""Implicit plotting module for SymPy.
+
+Explanation
+===========
 
 The module implements a data series called ImplicitSeries which is used by
 ``Plot`` class to plot implicit plots for different backends. The module,
@@ -11,26 +14,30 @@ algorithm.
 
 See Also
 ========
+
 sympy.plotting.plot
 
 References
 ==========
-- Jeffrey Allen Tupper. Reliable Two-Dimensional Graphing Methods for
+
+.. [1] Jeffrey Allen Tupper. Reliable Two-Dimensional Graphing Methods for
 Mathematical Formulae with Two Free Variables.
 
-- Jeffrey Allen Tupper. Graphing Equations with Generalized Interval
+.. [2] Jeffrey Allen Tupper. Graphing Equations with Generalized Interval
 Arithmetic. Master's thesis. University of Toronto, 1996
 
 """
 
-from __future__ import print_function, division
 
 from .plot import BaseSeries, Plot
 from .experimental_lambdify import experimental_lambdify, vectorized_lambdify
 from .intervalmath import interval
 from sympy.core.relational import (Equality, GreaterThan, LessThan,
                 Relational, StrictLessThan, StrictGreaterThan)
-from sympy import Eq, Tuple, sympify, Symbol, Dummy
+from sympy.core.containers import Tuple
+from sympy.core.relational import Eq
+from sympy.core.symbol import (Dummy, Symbol)
+from sympy.core.sympify import sympify
 from sympy.external import import_module
 from sympy.logic.boolalg import BooleanFunction
 from sympy.polys.polyutils import _sort_gens
@@ -46,8 +53,9 @@ class ImplicitSeries(BaseSeries):
     def __init__(self, expr, var_start_end_x, var_start_end_y,
             has_equality, use_interval_math, depth, nb_of_points,
             line_color):
-        super(ImplicitSeries, self).__init__()
+        super().__init__()
         self.expr = sympify(expr)
+        self.label = self.expr
         self.var_x = sympify(var_start_end_x[0])
         self.start_x = float(var_start_end_x[1])
         self.end_x = float(var_start_end_x[2])
@@ -84,7 +92,7 @@ class ImplicitSeries(BaseSeries):
             # AttributeError here.
             if self.use_interval_math:
                 warnings.warn("Adaptive meshing could not be applied to the"
-                            " expression. Using uniform meshing.")
+                            " expression. Using uniform meshing.", stacklevel=7)
             self.use_interval_math = False
 
         if self.use_interval_math:
@@ -262,7 +270,7 @@ def plot_implicit(expr, x_var=None, y_var=None, adaptive=True, depth=0,
         :format: doctest
         :include-source: True
 
-        >>> from sympy import plot_implicit, cos, sin, symbols, Eq, And
+        >>> from sympy import plot_implicit, symbols, Eq, And
         >>> x, y = symbols('x y')
 
     Without any ranges for the symbols in the expression:
@@ -416,8 +424,8 @@ def plot_implicit(expr, x_var=None, y_var=None, adaptive=True, depth=0,
     kwargs['xlim'] = tuple(float(x) for x in var_start_end_x[1:])
     kwargs['ylim'] = tuple(float(y) for y in var_start_end_y[1:])
     # set the x and y labels
-    kwargs.setdefault('xlabel', var_start_end_x[0].name)
-    kwargs.setdefault('ylabel', var_start_end_y[0].name)
+    kwargs.setdefault('xlabel', var_start_end_x[0])
+    kwargs.setdefault('ylabel', var_start_end_y[0])
     p = Plot(series_argument, **kwargs)
     if show:
         p.show()
