@@ -9,17 +9,46 @@ convenience routines for converting between Expr and the poly domains as well
 as unifying matrices with different domains.
 
 """
+from functools import reduce
+from typing import Union as tUnion, Tuple as tTuple
+
 from sympy.core.sympify import _sympify
 
+from ..domains import Domain
+
 from ..constructor import construct_domain
 
-from .exceptions import (NonSquareMatrixError, ShapeError, DDMShapeError,
-        DDMDomainError, DDMFormatError)
+from .exceptions import (DMNonSquareMatrixError, DMShapeError,
+                         DMDomainError, DMFormatError, DMBadInputError,
+                         DMNotAField)
 
 from .ddm import DDM
 
 from .sdm import SDM
 
+from .domainscalar import DomainScalar
+
+from sympy.polys.domains import ZZ, EXRAW
+
+
+def DM(rows, domain):
+    """Convenient alias for DomainMatrix.from_list
+
+    Examples
+    =======
+
+    >>> from sympy import ZZ
+    >>> from sympy.polys.matrices import DM
+    >>> DM([[1, 2], [3, 4]], ZZ)
+    DomainMatrix([[1, 2], [3, 4]], (2, 2), ZZ)
+
+    See also
+    =======
+
+    DomainMatrix.from_list
+    """
+    return DomainMatrix.from_list(rows, domain)
+
 
 class DomainMatrix:
     r"""
@@ -30,7 +59,7 @@ class DomainMatrix:
 
     DomainMatrix uses :py:class:`~.Domain` for its internal representation
     which makes it more faster for many common operations
-    than current sympy Matrix class, but this advantage makes it not
+    than current SymPy Matrix class, but this advantage makes it not
     entirely compatible with Matrix.
     DomainMatrix could be found analogous to numpy arrays with "dtype".
     In the DomainMatrix, each matrix has a domain such as :ref:`ZZ`
@@ -49,7 +78,7 @@ class DomainMatrix:
     ...    [3, 4]])
     >>> A = DomainMatrix.from_Matrix(Matrix1)
     >>> A
-    DomainMatrix([[1, 2], [3, 4]], (2, 2), ZZ)
+    DomainMatrix({0: {0: 1, 1: 2}, 1: {0: 3, 1: 4}}, (2, 2), ZZ)
 
     Driectly forming a DomainMatrix:
 
@@ -70,6 +99,9 @@ class DomainMatrix:
     Poly
 
     """
+    rep: tUnion[SDM, DDM]
+    shape: tTuple[int, int]
+    domain: Domain
 
     def __new__(cls, rows, shape, domain, *, fmt=None):
         """
@@ -109,6 +141,50 @@ class DomainMatrix:
 
         return cls.from_rep(rep)
 
+    def __getnewargs__(self):
+        rep = self.rep
+        if isinstance(rep, DDM):
+            arg = list(rep)
+        elif isinstance(rep, SDM):
+            arg = dict(rep)
+        else:
+            raise RuntimeError # pragma: no cover
+        return arg, self.shape, self.domain
+
+    def __getitem__(self, key):
+        i, j = key
+        m, n = self.shape
+        if not (isinstance(i, slice) or isinstance(j, slice)):
+            return DomainScalar(self.rep.getitem(i, j), self.domain)
+
+        if not isinstance(i, slice):
+            if not -m <= i < m:
+                raise IndexError("Row index out of range")
+            i = i % m
+            i = slice(i, i+1)
+        if not isinstance(j, slice):
+            if not -n <= j < n:
+                raise IndexError("Column index out of range")
+            j = j % n
+            j = slice(j, j+1)
+
+        return self.from_rep(self.rep.extract_slice(i, j))
+
+    def getitem_sympy(self, i, j):
+        return self.domain.to_sympy(self.rep.getitem(i, j))
+
+    def extract(self, rowslist, colslist):
+        return self.from_rep(self.rep.extract(rowslist, colslist))
+
+    def __setitem__(self, key, value):
+        i, j = key
+        if not self.domain.of_type(value):
+            raise TypeError
+        if isinstance(i, int) and isinstance(j, int):
+            self.rep.setitem(i, j, value)
+        else:
+            raise NotImplementedError
+
     @classmethod
     def from_rep(cls, rep):
         """Create a new DomainMatrix efficiently from DDM/SDM.
@@ -167,6 +243,53 @@ class DomainMatrix:
         self.domain = rep.domain
         return self
 
+
+    @classmethod
+    def from_list(cls, rows, domain):
+        r"""
+        Convert a list of lists into a DomainMatrix
+
+        Parameters
+        ==========
+
+        rows: list of lists
+            Each element of the inner lists should be either the single arg,
+            or tuple of args, that would be passed to the domain constructor
+            in order to form an element of the domain. See examples.
+
+        Returns
+        =======
+
+        DomainMatrix containing elements defined in rows
+
+        Examples
+        ========
+
+        >>> from sympy.polys.matrices import DomainMatrix
+        >>> from sympy import FF, QQ, ZZ
+        >>> A = DomainMatrix.from_list([[1, 0, 1], [0, 0, 1]], ZZ)
+        >>> A
+        DomainMatrix([[1, 0, 1], [0, 0, 1]], (2, 3), ZZ)
+        >>> B = DomainMatrix.from_list([[1, 0, 1], [0, 0, 1]], FF(7))
+        >>> B
+        DomainMatrix([[1 mod 7, 0 mod 7, 1 mod 7], [0 mod 7, 0 mod 7, 1 mod 7]], (2, 3), GF(7))
+        >>> C = DomainMatrix.from_list([[(1, 2), (3, 1)], [(1, 4), (5, 1)]], QQ)
+        >>> C
+        DomainMatrix([[1/2, 3], [1/4, 5]], (2, 2), QQ)
+
+        See Also
+        ========
+
+        from_list_sympy
+
+        """
+        nrows = len(rows)
+        ncols = 0 if not nrows else len(rows[0])
+        conv = lambda e: domain(*e) if isinstance(e, tuple) else domain(e)
+        domain_rows = [[conv(e) for e in row] for row in rows]
+        return DomainMatrix(domain_rows, (nrows, ncols), domain)
+
+
     @classmethod
     def from_list_sympy(cls, nrows, ncols, rows, **kwargs):
         r"""
@@ -188,14 +311,15 @@ class DomainMatrix:
         ========
 
         >>> from sympy.polys.matrices import DomainMatrix
-        >>> A = DomainMatrix.from_list_sympy(1, 2, [[1, 0]])
+        >>> from sympy.abc import x, y, z
+        >>> A = DomainMatrix.from_list_sympy(1, 3, [[x, y, z]])
         >>> A
-        DomainMatrix([[1, 0]], (1, 2), ZZ)
+        DomainMatrix([[x, y, z]], (1, 3), ZZ[x,y,z])
 
         See Also
         ========
 
-        sympy.polys.constructor.construct_domain
+        sympy.polys.constructor.construct_domain, from_dict_sympy
 
         """
         assert len(rows) == nrows
@@ -210,7 +334,57 @@ class DomainMatrix:
         return DomainMatrix(domain_rows, (nrows, ncols), domain)
 
     @classmethod
-    def from_Matrix(cls, M, **kwargs):
+    def from_dict_sympy(cls, nrows, ncols, elemsdict, **kwargs):
+        """
+
+        Parameters
+        ==========
+
+        nrows: number of rows
+        ncols: number of cols
+        elemsdict: dict of dicts containing non-zero elements of the DomainMatrix
+
+        Returns
+        =======
+
+        DomainMatrix containing elements of elemsdict
+
+        Examples
+        ========
+
+        >>> from sympy.polys.matrices import DomainMatrix
+        >>> from sympy.abc import x,y,z
+        >>> elemsdict = {0: {0:x}, 1:{1: y}, 2: {2: z}}
+        >>> A = DomainMatrix.from_dict_sympy(3, 3, elemsdict)
+        >>> A
+        DomainMatrix({0: {0: x}, 1: {1: y}, 2: {2: z}}, (3, 3), ZZ[x,y,z])
+
+        See Also
+        ========
+
+        from_list_sympy
+
+        """
+        if not all(0 <= r < nrows for r in elemsdict):
+            raise DMBadInputError("Row out of range")
+        if not all(0 <= c < ncols for row in elemsdict.values() for c in row):
+            raise DMBadInputError("Column out of range")
+
+        items_sympy = [_sympify(item) for row in elemsdict.values() for item in row.values()]
+        domain, items_domain = cls.get_domain(items_sympy, **kwargs)
+
+        idx = 0
+        items_dict = {}
+        for i, row in elemsdict.items():
+                items_dict[i] = {}
+                for j in row:
+                    items_dict[i][j] = items_domain[idx]
+                    idx += 1
+
+        return DomainMatrix(items_dict, (nrows, ncols), domain)
+
+    @classmethod
+    def from_Matrix(cls, M, fmt='sparse',**kwargs):
         r"""
         Convert Matrix to DomainMatrix
 
@@ -234,7 +408,14 @@ class DomainMatrix:
         ...    [2.4, 1]])
         >>> A = DomainMatrix.from_Matrix(M)
         >>> A
-        DomainMatrix([[1.0, 3.4], [2.4, 1.0]], (2, 2), RR)
+        DomainMatrix({0: {0: 1.0, 1: 3.4}, 1: {0: 2.4, 1: 1.0}}, (2, 2), RR)
+
+        We can keep internal representation as ddm using fmt='dense'
+        >>> from sympy import Matrix, QQ
+        >>> from sympy.polys.matrices import DomainMatrix
+        >>> A = DomainMatrix.from_Matrix(Matrix([[QQ(1, 2), QQ(3, 4)], [QQ(0, 1), QQ(0, 1)]]), fmt='dense')
+        >>> A.rep
+        [[1/2, 3/4], [0, 0]]
 
         See Also
         ========
@@ -242,13 +423,19 @@ class DomainMatrix:
         Matrix
 
         """
-        return cls.from_list_sympy(*M.shape, M.tolist(), **kwargs)
+        if fmt == 'dense':
+            return cls.from_list_sympy(*M.shape, M.tolist(), **kwargs)
+
+        return cls.from_dict_sympy(*M.shape, M.todod(), **kwargs)
 
     @classmethod
     def get_domain(cls, items_sympy, **kwargs):
         K, items_K = construct_domain(items_sympy, **kwargs)
         return K, items_K
 
+    def copy(self):
+        return self.from_rep(self.rep.copy())
+
     def convert_to(self, K):
         r"""
         Change the domain of DomainMatrix to desired domain or field
@@ -256,7 +443,9 @@ class DomainMatrix:
         Parameters
         ==========
 
-        K : Represents the desired domain or field
+        K : Represents the desired domain or field.
+            Alternatively, ``None`` may be passed, in which case this method
+            just returns a copy of this DomainMatrix.
 
         Returns
         =======
@@ -277,8 +466,13 @@ class DomainMatrix:
         DomainMatrix([[1, 2], [3, 4]], (2, 2), ZZ_I)
 
         """
+        if K is None:
+            return self.copy()
         return self.from_rep(self.rep.convert_to(K))
 
+    def to_sympy(self):
+        return self.convert_to(EXRAW)
+
     def to_field(self):
         r"""
         Returns a DomainMatrix with the appropriate field
@@ -348,36 +542,34 @@ class DomainMatrix:
 
         return self.from_rep(SDM.to_ddm(self.rep))
 
-    def _unify_domain(self, other):
-        """Convert self and other to a common domain"""
-        K1 = self.domain
-        K2 = other.domain
-        if K1 == K2:
-            return self, other
-        K = K1.unify(K2)
-        if K1 != K:
-            self = self.convert_to(K)
-        if K2 != K:
-            other = other.convert_to(K)
-        return self, other
+    @classmethod
+    def _unify_domain(cls, *matrices):
+        """Convert matrices to a common domain"""
+        domains = {matrix.domain for matrix in matrices}
+        if len(domains) == 1:
+            return matrices
+        domain = reduce(lambda x, y: x.unify(y), domains)
+        return tuple(matrix.convert_to(domain) for matrix in matrices)
 
-    def _unify_fmt(self, other, fmt):
-        """Convert self and other to the same format.
+    @classmethod
+    def _unify_fmt(cls, *matrices, fmt=None):
+        """Convert matrices to the same format.
 
-        If both are sparse or both are dense then return both unmodified.
+        If all matrices have the same format, then return unmodified.
         Otherwise convert both to the preferred format given as *fmt* which
         should be 'dense' or 'sparse'.
         """
-        if self.rep.fmt == other.rep.fmt:
-            return self, other
-        elif fmt == 'sparse':
-            return self.to_sparse(), other.to_sparse()
+        formats = {matrix.rep.fmt for matrix in matrices}
+        if len(formats) == 1:
+            return matrices
+        if fmt == 'sparse':
+            return tuple(matrix.to_sparse() for matrix in matrices)
         elif fmt == 'dense':
-            return self.to_dense(), other.to_dense()
+            return tuple(matrix.to_dense() for matrix in matrices)
         else:
             raise ValueError("fmt should be 'sparse' or 'dense'")
 
-    def unify(self, other, *, fmt=None):
+    def unify(self, *others, fmt=None):
         """
         Unifies the domains and the format of self and other
         matrices.
@@ -385,7 +577,8 @@ class DomainMatrix:
         Parameters
         ==========
 
-        other : another DomainMatrix
+        others : DomainMatrix
+
         fmt: string 'dense', 'sparse' or `None` (default)
             The preferred format to convert to if self and other are not
             already in the same format. If `None` or not specified then no
@@ -394,8 +587,8 @@ class DomainMatrix:
         Returns
         =======
 
-        (dM1, dM2)
-            dM1, dM2 DomainMatrix matrices with unified Domain and format
+        Tuple[DomainMatrix]
+            Matrices with unified domain and format
 
         Examples
         ========
@@ -429,11 +622,11 @@ class DomainMatrix:
         convert_to, to_dense, to_sparse
 
         """
-
-        dM1, dM2 = self._unify_domain(other)
+        matrices = (self,) + others
+        matrices = DomainMatrix._unify_domain(*matrices)
         if fmt is not None:
-            dM1, dM2 = dM1._unify_fmt(dM2, fmt)
-        return dM1, dM2
+            matrices = DomainMatrix._unify_fmt(*matrices, fmt=fmt)
+        return matrices
 
     def to_Matrix(self):
         r"""
@@ -467,46 +660,137 @@ class DomainMatrix:
         """
         from sympy.matrices.dense import MutableDenseMatrix
         elemlist = self.rep.to_list()
-        rows_sympy = [[self.domain.to_sympy(e) for e in row] for row in elemlist]
-        return MutableDenseMatrix(rows_sympy)
+        elements_sympy = [self.domain.to_sympy(e) for row in elemlist for e in row]
+        return MutableDenseMatrix(*self.shape, elements_sympy)
+
+    def to_list(self):
+        return self.rep.to_list()
+
+    def to_list_flat(self):
+        return self.rep.to_list_flat()
+
+    def to_dok(self):
+        return self.rep.to_dok()
 
     def __repr__(self):
         return 'DomainMatrix(%s, %r, %r)' % (str(self.rep), self.shape, self.domain)
 
-    def hstack(A, B):
-        r"""
-        Horizontally stacks 2 Domain Matrices.
+    def transpose(self):
+        """Matrix transpose of ``self``"""
+        return self.from_rep(self.rep.transpose())
+
+    def flat(self):
+        rows, cols = self.shape
+        return [self[i,j].element for i in range(rows) for j in range(cols)]
+
+    @property
+    def is_zero_matrix(self):
+        return self.rep.is_zero_matrix()
+
+    @property
+    def is_upper(self):
+        """
+        Says whether this matrix is upper-triangular. True can be returned
+        even if the matrix is not square.
+        """
+        return self.rep.is_upper()
+
+    @property
+    def is_lower(self):
+        """
+        Says whether this matrix is lower-triangular. True can be returned
+        even if the matrix is not square.
+        """
+        return self.rep.is_lower()
+
+    @property
+    def is_square(self):
+        return self.shape[0] == self.shape[1]
+
+    def rank(self):
+        rref, pivots = self.rref()
+        return len(pivots)
+
+    def hstack(A, *B):
+        r"""Horizontally stack the given matrices.
 
         Parameters
         ==========
 
-        A, B: DomainMatrix
-            to stack the rows horizontally
+        B: DomainMatrix
+            Matrices to stack horizontally.
 
         Returns
         =======
 
         DomainMatrix
-            DomainMatrix by stacking the rows horizontally
+            DomainMatrix by stacking horizontally.
 
         Examples
         ========
 
-        >>> from sympy import ZZ, QQ
+        >>> from sympy import ZZ
         >>> from sympy.polys.matrices import DomainMatrix
-        >>> A = DomainMatrix([[ZZ(1), ZZ(2), ZZ(3)]], (1, 3), ZZ)
-        >>> B = DomainMatrix([[QQ(-1, 2), QQ(1, 2), QQ(1, 3)]],(1, 3), QQ)
+
+        >>> A = DomainMatrix([[ZZ(1), ZZ(2)], [ZZ(3), ZZ(4)]], (2, 2), ZZ)
+        >>> B = DomainMatrix([[ZZ(5), ZZ(6)], [ZZ(7), ZZ(8)]], (2, 2), ZZ)
         >>> A.hstack(B)
-        DomainMatrix([[1, 2, 3, -1/2, 1/2, 1/3]], (1, 6), QQ)
+        DomainMatrix([[1, 2, 5, 6], [3, 4, 7, 8]], (2, 4), ZZ)
+
+        >>> C = DomainMatrix([[ZZ(9), ZZ(10)], [ZZ(11), ZZ(12)]], (2, 2), ZZ)
+        >>> A.hstack(B, C)
+        DomainMatrix([[1, 2, 5, 6, 9, 10], [3, 4, 7, 8, 11, 12]], (2, 6), ZZ)
 
         See Also
         ========
 
         unify
-
         """
-        A, B = A.unify(B, fmt='dense')
-        return A.from_rep(A.rep.hstack(B.rep))
+        A, *B = A.unify(*B, fmt='dense')
+        return DomainMatrix.from_rep(A.rep.hstack(*(Bk.rep for Bk in B)))
+
+    def vstack(A, *B):
+        r"""Vertically stack the given matrices.
+
+        Parameters
+        ==========
+
+        B: DomainMatrix
+            Matrices to stack vertically.
+
+        Returns
+        =======
+
+        DomainMatrix
+            DomainMatrix by stacking vertically.
+
+        Examples
+        ========
+
+        >>> from sympy import ZZ
+        >>> from sympy.polys.matrices import DomainMatrix
+
+        >>> A = DomainMatrix([[ZZ(1), ZZ(2)], [ZZ(3), ZZ(4)]], (2, 2), ZZ)
+        >>> B = DomainMatrix([[ZZ(5), ZZ(6)], [ZZ(7), ZZ(8)]], (2, 2), ZZ)
+        >>> A.vstack(B)
+        DomainMatrix([[1, 2], [3, 4], [5, 6], [7, 8]], (4, 2), ZZ)
+
+        >>> C = DomainMatrix([[ZZ(9), ZZ(10)], [ZZ(11), ZZ(12)]], (2, 2), ZZ)
+        >>> A.vstack(B, C)
+        DomainMatrix([[1, 2], [3, 4], [5, 6], [7, 8], [9, 10], [11, 12]], (6, 2), ZZ)
+
+        See Also
+        ========
+
+        unify
+        """
+        A, *B = A.unify(*B, fmt='dense')
+        return DomainMatrix.from_rep(A.rep.vstack(*(Bk.rep for Bk in B)))
+
+    def applyfunc(self, func, domain=None):
+        if domain is None:
+            domain = self.domain
+        return self.from_rep(self.rep.applyfunc(func, domain))
 
     def __add__(A, B):
         if not isinstance(B, DomainMatrix):
@@ -529,13 +813,19 @@ class DomainMatrix:
             A, B = A.unify(B, fmt='dense')
             return A.matmul(B)
         elif B in A.domain:
-            return A.from_rep(A.rep * B)
+            return A.scalarmul(B)
+        elif isinstance(B, DomainScalar):
+            A, B = A.unify(B)
+            return A.scalarmul(B.element)
         else:
             return NotImplemented
 
     def __rmul__(A, B):
         if B in A.domain:
-            return A.from_rep(A.rep * B)
+            return A.rscalarmul(B)
+        elif isinstance(B, DomainScalar):
+            A, B = A.unify(B)
+            return A.rscalarmul(B.element)
         else:
             return NotImplemented
 
@@ -548,13 +838,13 @@ class DomainMatrix:
     def _check(a, op, b, ashape, bshape):
         if a.domain != b.domain:
             msg = "Domain mismatch: %s %s %s" % (a.domain, op, b.domain)
-            raise DDMDomainError(msg)
+            raise DMDomainError(msg)
         if ashape != bshape:
             msg = "Shape mismatch: %s %s %s" % (a.shape, op, b.shape)
-            raise DDMShapeError(msg)
+            raise DMShapeError(msg)
         if a.rep.fmt != b.rep.fmt:
             msg = "Format mismatch: %s %s %s" % (a.rep.fmt, op, b.rep.fmt)
-            raise DDMFormatError(msg)
+            raise DMFormatError(msg)
 
     def add(A, B):
         r"""
@@ -575,7 +865,7 @@ class DomainMatrix:
         Raises
         ======
 
-        ShapeError
+        DMShapeError
             If the dimensions of the two DomainMatrix are not equal
 
         ValueError
@@ -625,7 +915,7 @@ class DomainMatrix:
         Raises
         ======
 
-        ShapeError
+        DMShapeError
             If the dimensions of the two DomainMatrix are not equal
 
         ValueError
@@ -729,6 +1019,9 @@ class DomainMatrix:
         """
         return A.from_rep(A.rep.mul(b))
 
+    def rmul(A, b):
+        return A.from_rep(A.rep.rmul(b))
+
     def matmul(A, B):
         r"""
         Performs matrix multiplication of two DomainMatrix matrices
@@ -770,6 +1063,42 @@ class DomainMatrix:
         A._check('*', B, A.shape[1], B.shape[0])
         return A.from_rep(A.rep.matmul(B.rep))
 
+    def _scalarmul(A, lamda, reverse):
+        if lamda == A.domain.zero:
+            return DomainMatrix.zeros(A.shape, A.domain)
+        elif lamda == A.domain.one:
+            return A.copy()
+        elif reverse:
+            return A.rmul(lamda)
+        else:
+            return A.mul(lamda)
+
+    def scalarmul(A, lamda):
+        return A._scalarmul(lamda, reverse=False)
+
+    def rscalarmul(A, lamda):
+        return A._scalarmul(lamda, reverse=True)
+
+    def mul_elementwise(A, B):
+        assert A.domain == B.domain
+        return A.from_rep(A.rep.mul_elementwise(B.rep))
+
+    def __truediv__(A, lamda):
+        """ Method for Scalar Divison"""
+        if isinstance(lamda, int) or ZZ.of_type(lamda):
+            lamda = DomainScalar(ZZ(lamda), ZZ)
+
+        if not isinstance(lamda, DomainScalar):
+            return NotImplemented
+
+        A, lamda = A.to_field().unify(lamda)
+        if lamda.element == lamda.domain.zero:
+            raise ZeroDivisionError
+        if lamda.element == lamda.domain.one:
+            return A.to_field()
+
+        return A.mul(1 / lamda.element)
+
     def pow(A, n):
         r"""
         Computes A**n
@@ -813,7 +1142,7 @@ class DomainMatrix:
         """
         nrows, ncols = A.shape
         if nrows != ncols:
-            raise NonSquareMatrixError('Power of a nonsquare matrix')
+            raise DMNonSquareMatrixError('Power of a nonsquare matrix')
         if n < 0:
             raise NotImplementedError('Negative powers')
         elif n == 0:
@@ -826,6 +1155,81 @@ class DomainMatrix:
             sqrtAn = A ** (n // 2)
             return sqrtAn * sqrtAn
 
+    def scc(self):
+        """Compute the strongly connected components of a DomainMatrix
+
+        Explanation
+        ===========
+
+        A square matrix can be considered as the adjacency matrix for a
+        directed graph where the row and column indices are the vertices. In
+        this graph if there is an edge from vertex ``i`` to vertex ``j`` if
+        ``M[i, j]`` is nonzero. This routine computes the strongly connected
+        components of that graph which are subsets of the rows and columns that
+        are connected by some nonzero element of the matrix. The strongly
+        connected components are useful because many operations such as the
+        determinant can be computed by working with the submatrices
+        corresponding to each component.
+
+        Examples
+        ========
+
+        Find the strongly connected components of a matrix:
+
+        >>> from sympy import ZZ
+        >>> from sympy.polys.matrices import DomainMatrix
+        >>> M = DomainMatrix([[ZZ(1), ZZ(0), ZZ(2)],
+        ...                   [ZZ(0), ZZ(3), ZZ(0)],
+        ...                   [ZZ(4), ZZ(6), ZZ(5)]], (3, 3), ZZ)
+        >>> M.scc()
+        [[1], [0, 2]]
+
+        Compute the determinant from the components:
+
+        >>> MM = M.to_Matrix()
+        >>> MM
+        Matrix([
+        [1, 0, 2],
+        [0, 3, 0],
+        [4, 6, 5]])
+        >>> MM[[1], [1]]
+        Matrix([[3]])
+        >>> MM[[0, 2], [0, 2]]
+        Matrix([
+        [1, 2],
+        [4, 5]])
+        >>> MM.det()
+        -9
+        >>> MM[[1], [1]].det() * MM[[0, 2], [0, 2]].det()
+        -9
+
+        The components are given in reverse topological order and represent a
+        permutation of the rows and columns that will bring the matrix into
+        block lower-triangular form:
+
+        >>> MM[[1, 0, 2], [1, 0, 2]]
+        Matrix([
+        [3, 0, 0],
+        [0, 1, 2],
+        [6, 4, 5]])
+
+        Returns
+        =======
+
+        List of lists of integers
+            Each list represents a strongly connected component.
+
+        See also
+        ========
+
+        sympy.matrices.matrices.MatrixBase.strongly_connected_components
+        sympy.utilities.iterables.strongly_connected_components
+
+        """
+        rows, cols = self.shape
+        assert rows == cols
+        return self.rep.scc()
+
     def rref(self):
         r"""
         Returns reduced-row echelon form and list of pivots for the DomainMatrix
@@ -865,19 +1269,75 @@ class DomainMatrix:
 
         """
         if not self.domain.is_Field:
-            raise ValueError('Not a field')
+            raise DMNotAField('Not a field')
         rref_ddm, pivots = self.rep.rref()
         return self.from_rep(rref_ddm), tuple(pivots)
 
-    def nullspace(self):
+    def columnspace(self):
         r"""
-        Returns the Null Space for the DomainMatrix
+        Returns the columnspace for the DomainMatrix
 
         Returns
         =======
 
         DomainMatrix
-            Null Space of the DomainMatrix
+            The columns of this matrix form a basis for the columnspace.
+
+        Examples
+        ========
+
+        >>> from sympy import QQ
+        >>> from sympy.polys.matrices import DomainMatrix
+        >>> A = DomainMatrix([
+        ...    [QQ(1), QQ(-1)],
+        ...    [QQ(2), QQ(-2)]], (2, 2), QQ)
+        >>> A.columnspace()
+        DomainMatrix([[1], [2]], (2, 1), QQ)
+
+        """
+        if not self.domain.is_Field:
+            raise DMNotAField('Not a field')
+        rref, pivots = self.rref()
+        rows, cols = self.shape
+        return self.extract(range(rows), pivots)
+
+    def rowspace(self):
+        r"""
+        Returns the rowspace for the DomainMatrix
+
+        Returns
+        =======
+
+        DomainMatrix
+            The rows of this matrix form a basis for the rowspace.
+
+        Examples
+        ========
+
+        >>> from sympy import QQ
+        >>> from sympy.polys.matrices import DomainMatrix
+        >>> A = DomainMatrix([
+        ...    [QQ(1), QQ(-1)],
+        ...    [QQ(2), QQ(-2)]], (2, 2), QQ)
+        >>> A.rowspace()
+        DomainMatrix([[1, -1]], (1, 2), QQ)
+
+        """
+        if not self.domain.is_Field:
+            raise DMNotAField('Not a field')
+        rref, pivots = self.rref()
+        rows, cols = self.shape
+        return self.extract(range(len(pivots)), range(cols))
+
+    def nullspace(self):
+        r"""
+        Returns the nullspace for the DomainMatrix
+
+        Returns
+        =======
+
+        DomainMatrix
+            The rows of this matrix form a basis for the nullspace.
 
         Examples
         ========
@@ -891,6 +1351,8 @@ class DomainMatrix:
         DomainMatrix([[1, 1]], (1, 2), QQ)
 
         """
+        if not self.domain.is_Field:
+            raise DMNotAField('Not a field')
         return self.from_rep(self.rep.nullspace()[0])
 
     def inv(self):
@@ -909,7 +1371,7 @@ class DomainMatrix:
         ValueError
             If the domain of DomainMatrix not a Field
 
-        NonSquareMatrixError
+        DMNonSquareMatrixError
             If the DomainMatrix is not a not Square DomainMatrix
 
         Examples
@@ -931,10 +1393,10 @@ class DomainMatrix:
 
         """
         if not self.domain.is_Field:
-            raise ValueError('Not a field')
+            raise DMNotAField('Not a field')
         m, n = self.shape
         if m != n:
-            raise NonSquareMatrixError
+            raise DMNonSquareMatrixError
         inv = self.rep.inv()
         return self.from_rep(inv)
 
@@ -969,7 +1431,7 @@ class DomainMatrix:
         """
         m, n = self.shape
         if m != n:
-            raise NonSquareMatrixError
+            raise DMNonSquareMatrixError
         return self.rep.det()
 
     def lu(self):
@@ -1007,7 +1469,7 @@ class DomainMatrix:
 
         """
         if not self.domain.is_Field:
-            raise ValueError('Not a field')
+            raise DMNotAField('Not a field')
         L, U, swaps = self.rep.lu()
         return self.from_rep(L), self.from_rep(U), swaps
 
@@ -1029,7 +1491,7 @@ class DomainMatrix:
         Raises
         ======
 
-        ShapeError
+        DMShapeError
             If the DomainMatrix A and rhs have different number of rows
 
         ValueError
@@ -1057,12 +1519,26 @@ class DomainMatrix:
 
         """
         if self.shape[0] != rhs.shape[0]:
-            raise ShapeError("Shape")
+            raise DMShapeError("Shape")
         if not self.domain.is_Field:
-            raise ValueError('Not a field')
+            raise DMNotAField('Not a field')
         sol = self.rep.lu_solve(rhs.rep)
         return self.from_rep(sol)
 
+    def _solve(A, b):
+        # XXX: Not sure about this method or its signature. It is just created
+        # because it is needed by the holonomic module.
+        if A.shape[0] != b.shape[0]:
+            raise DMShapeError("Shape")
+        if A.domain != b.domain or not A.domain.is_Field:
+            raise DMNotAField('Not a field')
+        Aaug = A.hstack(b)
+        Arref, pivots = Aaug.rref()
+        particular = Arref.from_rep(Arref.rep.particular())
+        nullspace_rep, nonpivots = Arref[:,:-1].rep.nullspace()
+        nullspace = Arref.from_rep(nullspace_rep)
+        return particular, nullspace
+
     def charpoly(self):
         r"""
         Returns the coefficients of the characteristic polynomial
@@ -1078,7 +1554,7 @@ class DomainMatrix:
         Raises
         ======
 
-        NonSquareMatrixError
+        DMNonSquareMatrixError
             If the DomainMatrix is not a not Square DomainMatrix
 
         Examples
@@ -1096,11 +1572,11 @@ class DomainMatrix:
         """
         m, n = self.shape
         if m != n:
-            raise NonSquareMatrixError("not square")
+            raise DMNonSquareMatrixError("not square")
         return self.rep.charpoly()
 
     @classmethod
-    def eye(cls, n, domain):
+    def eye(cls, shape, domain):
         r"""
         Return identity matrix of size n
 
@@ -1113,7 +1589,28 @@ class DomainMatrix:
         DomainMatrix({0: {0: 1}, 1: {1: 1}, 2: {2: 1}}, (3, 3), QQ)
 
         """
-        return cls.from_rep(SDM.eye(n, domain))
+        if isinstance(shape, int):
+            shape = (shape, shape)
+        return cls.from_rep(SDM.eye(shape, domain))
+
+    @classmethod
+    def diag(cls, diagonal, domain, shape=None):
+        r"""
+        Return diagonal matrix with entries from ``diagonal``.
+
+        Examples
+        ========
+
+        >>> from sympy.polys.matrices import DomainMatrix
+        >>> from sympy import ZZ
+        >>> DomainMatrix.diag([ZZ(5), ZZ(6)], ZZ)
+        DomainMatrix({0: {0: 5}, 1: {1: 6}}, (2, 2), ZZ)
+
+        """
+        if shape is None:
+            N = len(diagonal)
+            shape = (N, N)
+        return cls.from_rep(SDM.diag(diagonal, domain, shape))
 
     @classmethod
     def zeros(cls, shape, domain, *, fmt='sparse'):
@@ -1130,6 +1627,21 @@ class DomainMatrix:
         """
         return cls.from_rep(SDM.zeros(shape, domain))
 
+    @classmethod
+    def ones(cls, shape, domain):
+        """Returns a DomainMatrix of 1s, of size shape, belonging to the specified domain
+
+        Examples
+        ========
+
+        >>> from sympy.polys.matrices import DomainMatrix
+        >>> from sympy import QQ
+        >>> DomainMatrix.ones((2,3), QQ)
+        DomainMatrix([[1, 1, 1], [1, 1, 1]], (2, 3), QQ)
+
+        """
+        return cls.from_rep(DDM.ones(shape, domain))
+
     def __eq__(A, B):
         r"""
         Checks for two DomainMatrix matrices to be equal or not
@@ -1169,6 +1681,13 @@ class DomainMatrix:
         False
 
         """
-        if not isinstance(B, DomainMatrix):
+        if not isinstance(A, type(B)):
             return NotImplemented
-        return A.rep == B.rep
+        return A.domain == B.domain and A.rep == B.rep
+
+    def unify_eq(A, B):
+        if A.shape != B.shape:
+            return False
+        if A.domain != B.domain:
+            A, B = A.unify(B)
+        return A == B
