@@ -1,20 +1,20 @@
 """
-A Printer for generating readable representation of most sympy classes.
+A Printer for generating readable representation of most SymPy classes.
 """
 
-from __future__ import print_function, division
+from typing import Any, Dict as tDict
 
-from typing import Any, Dict
-
-from sympy.core import S, Rational, Pow, Basic, Mul
+from sympy.core import S, Rational, Pow, Basic, Mul, Number, Add
 from sympy.core.mul import _keep_coeff
-from .printer import Printer
-from sympy.printing.precedence import precedence, PRECEDENCE
+from sympy.core.relational import Relational
+from sympy.core.sorting import default_sort_key
+from sympy.core.sympify import SympifyError
+from sympy.utilities.iterables import sift
+from .precedence import precedence, PRECEDENCE
+from .printer import Printer, print_function
 
 from mpmath.libmp import prec_to_dps, to_str as mlib_to_str
 
-from sympy.utilities import default_sort_key
-
 
 class StrPrinter(Printer):
     printmethod = "_sympystr"
@@ -26,9 +26,9 @@ class StrPrinter(Printer):
         "perm_cyclic": True,
         "min": None,
         "max": None,
-    }  # type: Dict[str, Any]
+    }  # type: tDict[str, Any]
 
-    _relationals = dict()  # type: Dict[str, str]
+    _relationals = {}  # type: tDict[str, str]
 
     def parenthesize(self, item, level, strict=False):
         if (precedence(item) < level) or ((not strict) and precedence(item) <= level):
@@ -59,7 +59,7 @@ class StrPrinter(Printer):
                 t = t[1:]
             else:
                 sign = "+"
-            if precedence(term) < PREC:
+            if precedence(term) < PREC or isinstance(term, Add):
                 l.extend([sign, "(%s)" % t])
             else:
                 l.extend([sign, t])
@@ -78,7 +78,12 @@ class StrPrinter(Printer):
         return '~%s' %(self.parenthesize(expr.args[0],PRECEDENCE["Not"]))
 
     def _print_And(self, expr):
-        return self.stringify(expr.args, " & ", PRECEDENCE["BitwiseAnd"])
+        args = list(expr.args)
+        for j, i in enumerate(args):
+            if isinstance(i, Relational) and (
+                    i.canonical.rhs is S.NegativeInfinity):
+                args.insert(0, args.pop(j))
+        return self.stringify(args, " & ", PRECEDENCE["BitwiseAnd"])
 
     def _print_Or(self, expr):
         return self.stringify(expr.args, " | ", PRECEDENCE["BitwiseOr"])
@@ -87,7 +92,8 @@ class StrPrinter(Printer):
         return self.stringify(expr.args, " ^ ", PRECEDENCE["BitwiseXor"])
 
     def _print_AppliedPredicate(self, expr):
-        return '%s(%s)' % (self._print(expr.func), self._print(expr.arg))
+        return '%s(%s)' % (
+            self._print(expr.function), self.stringify(expr.arguments, ", "))
 
     def _print_Basic(self, expr):
         l = [self._print(o) for o in expr.args]
@@ -156,6 +162,11 @@ class StrPrinter(Printer):
     def _print_GoldenRatio(self, expr):
         return 'GoldenRatio'
 
+    def _print_Heaviside(self, expr):
+        # Same as _print_Function but uses pargs to suppress default 1/2 for
+        # 2nd args
+        return expr.func.__name__ + "(%s)" % self.stringify(expr.pargs, ", ")
+
     def _print_TribonacciConstant(self, expr):
         return 'TribonacciConstant'
 
@@ -222,34 +233,12 @@ class StrPrinter(Printer):
     def _print_list(self, expr):
         return "[%s]" % self.stringify(expr, ", ")
 
+    def _print_List(self, expr):
+        return self._print_list(expr)
+
     def _print_MatrixBase(self, expr):
         return expr._format_str(self)
 
-    def _print_MutableSparseMatrix(self, expr):
-        return self._print_MatrixBase(expr)
-
-    def _print_SparseMatrix(self, expr):
-        from sympy.matrices import Matrix
-        return self._print(Matrix(expr))
-
-    def _print_ImmutableSparseMatrix(self, expr):
-        return self._print_MatrixBase(expr)
-
-    def _print_Matrix(self, expr):
-        return self._print_MatrixBase(expr)
-
-    def _print_DenseMatrix(self, expr):
-        return self._print_MatrixBase(expr)
-
-    def _print_MutableDenseMatrix(self, expr):
-        return self._print_MatrixBase(expr)
-
-    def _print_ImmutableMatrix(self, expr):
-        return self._print_MatrixBase(expr)
-
-    def _print_ImmutableDenseMatrix(self, expr):
-        return self._print_MatrixBase(expr)
-
     def _print_MatrixElement(self, expr):
         return self.parenthesize(expr.parent, PRECEDENCE["Atom"], strict=True) \
             + '[%s, %s]' % (self._print(expr.i), self._print(expr.j))
@@ -275,6 +264,53 @@ class StrPrinter(Printer):
 
         prec = precedence(expr)
 
+        # Check for unevaluated Mul. In this case we need to make sure the
+        # identities are visible, multiple Rational factors are not combined
+        # etc so we display in a straight-forward form that fully preserves all
+        # args and their order.
+        args = expr.args
+        if args[0] is S.One or any(
+                isinstance(a, Number) or
+                a.is_Pow and all(ai.is_Integer for ai in a.args)
+                for a in args[1:]):
+            d, n = sift(args, lambda x:
+                isinstance(x, Pow) and bool(x.exp.as_coeff_Mul()[0] < 0),
+                binary=True)
+            for i, di in enumerate(d):
+                if di.exp.is_Number:
+                    e = -di.exp
+                else:
+                    dargs = list(di.exp.args)
+                    dargs[0] = -dargs[0]
+                    e = Mul._from_args(dargs)
+                d[i] = Pow(di.base, e, evaluate=False) if e - 1 else di.base
+
+            pre = []
+            # don't parenthesize first factor if negative
+            if n and n[0].could_extract_minus_sign():
+                pre = [str(n.pop(0))]
+
+            nfactors = pre + [self.parenthesize(a, prec, strict=False)
+                for a in n]
+            if not nfactors:
+                nfactors = ['1']
+
+            # don't parenthesize first of denominator unless singleton
+            if len(d) > 1 and d[0].could_extract_minus_sign():
+                pre = [str(d.pop(0))]
+            else:
+                pre = []
+            dfactors = pre + [self.parenthesize(a, prec, strict=False)
+                for a in d]
+
+            n = '*'.join(nfactors)
+            d = '*'.join(dfactors)
+            if len(dfactors) > 1:
+                return '%s/(%s)' % (n, d)
+            elif dfactors:
+                return '%s/%s' % (n, d)
+            return n
+
         c, e = expr.as_coeff_Mul()
         if c < 0:
             expr = _keep_coeff(-c, e)
@@ -294,14 +330,29 @@ class StrPrinter(Printer):
             args = Mul.make_args(expr)
 
         # Gather args for numerator/denominator
+        def apow(i):
+            b, e = i.as_base_exp()
+            eargs = list(Mul.make_args(e))
+            if eargs[0] is S.NegativeOne:
+                eargs = eargs[1:]
+            else:
+                eargs[0] = -eargs[0]
+            e = Mul._from_args(eargs)
+            if isinstance(i, Pow):
+                return i.func(b, e, evaluate=False)
+            return i.func(e, evaluate=False)
         for item in args:
-            if item.is_commutative and item.is_Pow and item.exp.is_Rational and item.exp.is_negative:
-                if item.exp != -1:
-                    b.append(Pow(item.base, -item.exp, evaluate=False))
+            if (item.is_commutative and
+                    isinstance(item, Pow) and
+                    bool(item.exp.as_coeff_Mul()[0] < 0)):
+                if item.exp is not S.NegativeOne:
+                    b.append(apow(item))
                 else:
-                    if len(item.args[0].args) != 1 and isinstance(item.base, Mul):   # To avoid situations like #14160
+                    if (len(item.args[0].args) != 1 and
+                            isinstance(item.base, (Mul, Pow))):
+                        # To avoid situations like #14160
                         pow_paren.append(item)
-                    b.append(Pow(item.base, -item.exp))
+                    b.append(item.base)
             elif item.is_Rational and item is not S.Infinity:
                 if item.p != 1:
                     a.append(Rational(item.p))
@@ -345,7 +396,7 @@ class StrPrinter(Printer):
         )
 
     def _print_ElementwiseApplyFunction(self, expr):
-        return "{0}.({1})".format(
+        return "{}.({})".format(
             expr.function,
             self._print(expr.expr),
         )
@@ -373,16 +424,19 @@ class StrPrinter(Printer):
 
     def _print_Permutation(self, expr):
         from sympy.combinatorics.permutations import Permutation, Cycle
-        from sympy.utilities.exceptions import SymPyDeprecationWarning
+        from sympy.utilities.exceptions import sympy_deprecation_warning
 
         perm_cyclic = Permutation.print_cyclic
         if perm_cyclic is not None:
-            SymPyDeprecationWarning(
-                feature="Permutation.print_cyclic = {}".format(perm_cyclic),
-                useinstead="init_printing(perm_cyclic={})"
-                .format(perm_cyclic),
-                issue=15201,
-                deprecated_since_version="1.6").warn()
+            sympy_deprecation_warning(
+                f"""
+                Setting Permutation.print_cyclic is deprecated. Instead use
+                init_printing(perm_cyclic={perm_cyclic}).
+                """,
+                deprecated_since_version="1.6",
+                active_deprecations_target="deprecated-permutation-print_cyclic",
+                stacklevel=7,
+            )
         else:
             perm_cyclic = self._settings.get("perm_cyclic", True)
 
@@ -436,6 +490,13 @@ class StrPrinter(Printer):
     def _print_TensAdd(self, expr):
         return expr._print()
 
+    def _print_ArraySymbol(self, expr):
+        return self._print(expr.name)
+
+    def _print_ArrayElement(self, expr):
+        return "%s[%s]" % (
+            self.parenthesize(expr.name, PRECEDENCE["Func"], True), ", ".join([self._print(i) for i in expr.indices]))
+
     def _print_PermutationGroup(self, expr):
         p = ['    %s' % self._print(a) for a in expr.args]
         return 'PermutationGroup([\n%s])' % ',\n'.join(p)
@@ -456,6 +517,9 @@ class StrPrinter(Printer):
     def _print_FreeGroupElement(self, elm):
         return elm.__str__()
 
+    def _print_GaussianElement(self, poly):
+        return "(%s + %s*I)" % (poly.x, poly.y)
+
     def _print_PolyElement(self, poly):
         return poly.str(self, PRECEDENCE, "%s**%s", "*")
 
@@ -474,12 +538,12 @@ class StrPrinter(Printer):
         for monom, coeff in expr.terms():
             s_monom = []
 
-            for i, exp in enumerate(monom):
-                if exp > 0:
-                    if exp == 1:
+            for i, e in enumerate(monom):
+                if e > 0:
+                    if e == 1:
                         s_monom.append(gens[i])
                     else:
-                        s_monom.append(gens[i] + "**%d" % exp)
+                        s_monom.append(gens[i] + "**%d" % e)
 
             s_monom = "*".join(s_monom)
 
@@ -510,7 +574,7 @@ class StrPrinter(Printer):
             else:
                 terms.extend(['+', s_term])
 
-        if terms[0] in ['-', '+']:
+        if terms[0] in ('-', '+'):
             modifier = terms.pop(0)
 
             if modifier == '-':
@@ -558,8 +622,7 @@ class StrPrinter(Printer):
         Examples
         ========
 
-        >>> from sympy.functions import sqrt
-        >>> from sympy.printing.str import StrPrinter
+        >>> from sympy import sqrt, StrPrinter
         >>> from sympy.abc import x
 
         How ``rational`` keyword works with ``sqrt``:
@@ -612,12 +675,6 @@ class StrPrinter(Printer):
         return '%s**%s' % (self.parenthesize(expr.base, PREC, strict=False),
                          self.parenthesize(expr.exp, PREC, strict=False))
 
-    def _print_ImmutableDenseNDimArray(self, expr):
-        return str(expr)
-
-    def _print_ImmutableSparseNDimArray(self, expr):
-        return str(expr)
-
     def _print_Integer(self, expr):
         if self._settings.get("sympy_integers", False):
             return "S(%s)" % (expr)
@@ -758,6 +815,21 @@ class StrPrinter(Printer):
             return "set()"
         return '{%s}' % args
 
+    def _print_FiniteSet(self, s):
+        from sympy.sets.sets import FiniteSet
+        items = sorted(s, key=default_sort_key)
+
+        args = ', '.join(self._print(item) for item in items)
+        if any(item.has(FiniteSet) for item in items):
+            return 'FiniteSet({})'.format(args)
+        return '{{{}}}'.format(args)
+
+    def _print_Partition(self, s):
+        items = sorted(s, key=default_sort_key)
+
+        args = ', '.join(self._print(arg) for arg in items)
+        return 'Partition({})'.format(args)
+
     def _print_frozenset(self, s):
         if not s:
             return "frozenset()"
@@ -826,13 +898,21 @@ class StrPrinter(Printer):
     def _print_WildFunction(self, expr):
         return expr.name + '_'
 
+    def _print_WildDot(self, expr):
+        return expr.name
+
+    def _print_WildPlus(self, expr):
+        return expr.name
+
+    def _print_WildStar(self, expr):
+        return expr.name
+
     def _print_Zero(self, expr):
         if self._settings.get("sympy_integers", False):
             return "S(0)"
         return "0"
 
     def _print_DMP(self, p):
-        from sympy.core.sympify import SympifyError
         try:
             if p.ring is not None:
                 # TODO incorporate order
@@ -863,16 +943,25 @@ class StrPrinter(Printer):
     def _print_Category(self, category):
         return 'Category("%s")' % category.name
 
+    def _print_Manifold(self, manifold):
+        return manifold.name.name
+
+    def _print_Patch(self, patch):
+        return patch.name.name
+
+    def _print_CoordSystem(self, coords):
+        return coords.name.name
+
     def _print_BaseScalarField(self, field):
-        return field._coord_sys._names[field._index]
+        return field._coord_sys.symbols[field._index].name
 
     def _print_BaseVectorField(self, field):
-        return 'e_%s' % field._coord_sys._names[field._index]
+        return 'e_%s' % field._coord_sys.symbols[field._index].name
 
     def _print_Differential(self, diff):
         field = diff._form_field
         if hasattr(field, '_coord_sys'):
-            return 'd%s' % field._coord_sys._names[field._index]
+            return 'd%s' % field._coord_sys.symbols[field._index].name
         else:
             return 'd(%s)' % self._print(field)
 
@@ -880,7 +969,17 @@ class StrPrinter(Printer):
         #TODO : Handle indices
         return "%s(%s)" % ("Tr", self._print(expr.args[0]))
 
+    def _print_Str(self, s):
+        return self._print(s.name)
 
+    def _print_AppliedBinaryRelation(self, expr):
+        rel = expr.function
+        return '%s(%s, %s)' % (self._print(rel),
+                               self._print(expr.lhs),
+                               self._print(expr.rhs))
+
+
+@print_function(StrPrinter)
 def sstr(expr, **settings):
     """Returns the expression as a string.
 
@@ -909,7 +1008,12 @@ class StrReprPrinter(StrPrinter):
     def _print_str(self, s):
         return repr(s)
 
+    def _print_Str(self, s):
+        # Str does not to be printed same as str here
+        return "%s(%s)" % (s.__class__.__name__, self._print(s.name))
 
+
+@print_function(StrReprPrinter)
 def sstrrepr(expr, **settings):
     """return expr in mixed str/repr form
 
