@@ -1,6 +1,14 @@
-from sympy import (Symbol, Rational, Order, exp, ln, log, nan, oo, O, pi, I,
-    S, Integral, sin, cos, sqrt, conjugate, expand, transpose, symbols,
-    Function, Add)
+from sympy.core.add import Add
+from sympy.core.function import (Function, expand)
+from sympy.core.numbers import (I, Rational, nan, oo, pi)
+from sympy.core.singleton import S
+from sympy.core.symbol import (Symbol, symbols)
+from sympy.functions.elementary.complexes import (conjugate, transpose)
+from sympy.functions.elementary.exponential import (exp, log)
+from sympy.functions.elementary.miscellaneous import sqrt
+from sympy.functions.elementary.trigonometric import (cos, sin)
+from sympy.integrals.integrals import Integral
+from sympy.series.order import O, Order
 from sympy.core.expr import unchanged
 from sympy.testing.pytest import raises
 from sympy.abc import w, x, y, z
@@ -45,7 +53,7 @@ def test_simple_2():
     assert Order(2*x)*x == Order(x**2)
     assert Order(2*x)/x == Order(1, x)
     assert Order(2*x)*x*exp(1/x) == Order(x**2*exp(1/x))
-    assert (Order(2*x)*x*exp(1/x)/ln(x)**3).expr == x**2*exp(1/x)*ln(x)**-3
+    assert (Order(2*x)*x*exp(1/x)/log(x)**3).expr == x**2*exp(1/x)*log(x)**-3
 
 
 def test_simple_3():
@@ -134,8 +142,8 @@ def test_contains_3():
 
 
 def test_contains_4():
-    assert Order(sin(1/x**2)).contains(Order(cos(1/x**2))) is None
-    assert Order(cos(1/x**2)).contains(Order(sin(1/x**2))) is None
+    assert Order(sin(1/x**2)).contains(Order(cos(1/x**2))) is True
+    assert Order(cos(1/x**2)).contains(Order(sin(1/x**2))) is True
 
 
 def test_contains():
@@ -149,7 +157,11 @@ def test_add_1():
     assert Order(3*x - 2*x**2) == Order(x)
     assert Order(1 + x) == Order(1, x)
     assert Order(1 + 1/x) == Order(1/x)
-    assert Order(ln(x) + 1/ln(x)) == Order(ln(x))
+    # TODO : A better output for Order(log(x) + 1/log(x))
+    # could be Order(log(x)). Currently Order for expressions
+    # where all arguments would involve a log term would fall
+    # in this category and outputs for these should be improved.
+    assert Order(log(x) + 1/log(x)) == Order((log(x)**2 + 1)/log(x))
     assert Order(exp(1/x) + x) == Order(exp(1/x))
     assert Order(exp(1/x) + 1/x**20) == Order(exp(1/x))
 
@@ -157,7 +169,7 @@ def test_add_1():
 def test_ln_args():
     assert O(log(x)) + O(log(2*x)) == O(log(x))
     assert O(log(x)) + O(log(x**3)) == O(log(x))
-    assert O(log(x*y)) + O(log(x) + log(y)) == O(log(x*y))
+    assert O(log(x*y)) + O(log(x) + log(y)) == O(log(x) + log(y), x, y)
 
 
 def test_multivar_0():
@@ -168,11 +180,11 @@ def test_multivar_0():
     assert Order(x*y*z).expr == x*y*z
     assert Order(x/y).expr == x/y
     assert Order(x*exp(1/y)).expr == x*exp(1/y)
-    assert Order(exp(x)*exp(1/y)).expr == exp(1/y)
+    assert Order(exp(x)*exp(1/y)).expr == exp(x)*exp(1/y)
 
 
 def test_multivar_0a():
-    assert Order(exp(1/x)*exp(1/y)).expr == exp(1/x + 1/y)
+    assert Order(exp(1/x)*exp(1/y)).expr == exp(1/x)*exp(1/y)
 
 
 def test_multivar_1():
@@ -277,7 +289,7 @@ def test_getO():
 
 
 def test_leading_term():
-    from sympy import digamma
+    from sympy.functions.special.gamma_functions import digamma
     assert O(1/digamma(1/x)) == O(1/log(x))
 
 
@@ -349,7 +361,7 @@ def test_order_at_infinity():
     assert Order(2*x, (x, oo))*x == Order(x**2, (x, oo))
     assert Order(2*x, (x, oo))/x == Order(1, (x, oo))
     assert Order(2*x, (x, oo))*x*exp(1/x) == Order(x**2*exp(1/x), (x, oo))
-    assert Order(2*x, (x, oo))*x*exp(1/x)/ln(x)**3 == Order(x**2*exp(1/x)*ln(x)**-3, (x, oo))
+    assert Order(2*x, (x, oo))*x*exp(1/x)/log(x)**3 == Order(x**2*exp(1/x)*log(x)**-3, (x, oo))
 
     assert Order(x, (x, oo)) + 1/x == 1/x + Order(x, (x, oo)) == Order(x, (x, oo))
     assert Order(x, (x, oo)) + 1 == 1 + Order(x, (x, oo)) == Order(x, (x, oo))
@@ -374,7 +386,7 @@ def test_order_at_infinity():
 
     # issue 7207
     assert Order(exp(x), (x, oo)).expr == Order(2*exp(x), (x, oo)).expr == exp(x)
-    assert Order(y**x, (x, oo)).expr == Order(2*y**x, (x, oo)).expr == exp(log(y)*x)
+    assert Order(y**x, (x, oo)).expr == Order(2*y**x, (x, oo)).expr == exp(x*log(y))
 
     # issue 19545
     assert Order(1/x - 3/(3*x + 2), (x, oo)).expr == x**(-2)
@@ -446,3 +458,20 @@ def test_issue_15539():
 
 def test_issue_18606():
     assert unchanged(Order, 0)
+
+
+def test_issue_22165():
+    assert O(log(x)).contains(2)
+
+
+def test_issue_23231():
+    # This test checks Order for expressions having
+    # arguments containing variables in exponents/powers.
+    assert O(x**x + 2**x, (x, oo)) == O(exp(x*log(x)), (x, oo))
+    assert O(x**x + x**2, (x, oo)) == O(exp(x*log(x)), (x, oo))
+    assert O(x**x + 1/x**2, (x, oo)) == O(exp(x*log(x)), (x, oo))
+    assert O(2**x + 3**x , (x, oo)) == O(exp(x*log(3)), (x, oo))
+
+
+def test_issue_9917():
+    assert O(x*sin(x) + 1, (x, oo)) == O(x, (x, oo))
